# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import django
import pytest
from captcha.models import CaptchaStore
from django.conf import settings
from django.core import mail
from django.core.management import call_command
from django.test import TestCase
from django.urls import reverse

from karaage.people.models import Person
from karaage.tests import fixtures

from ..models import Applicant, Application, ProjectApplication
from .fixtures import (
    ExistingProjectApplicationFactory,
    NewProjectApplicationFactory,
)


def set_admin():
    settings.ADMIN_IGNORED = False


def set_no_admin():
    settings.ADMIN_IGNORED = True


@pytest.mark.django_db
class UserApplicationTestCase(TestCase):

    def setUp(self):
        call_command('loaddata', 'test_karaage', **{'verbosity': 0})

    def tearDown(self):
        set_admin()

    def test_register_account(self):
        if django.VERSION >= (1, 9):
            url_prefix = ""
        else:
            url_prefix = 'http://testserver'

        set_no_admin()

        with self.assertRaises(Person.DoesNotExist):
            Person.objects.get(username='jimbob')

        self.assertEqual(len(mail.outbox), 0)
        response = self.client.get(reverse('kg_application_new'))
        self.assertEqual(response.status_code, 200)
        a = response.content.find(
            b'name="captcha_0" value="') + 24
        b = a + 40
        hash_ = response.content[a:b].decode("ascii")

        captcha_text = CaptchaStore.objects.get(hashkey=hash_).response

        # OPEN APPLICATION
        form_data = {
            'email': 'jim.bob@example.com',
            'captcha_0': hash_,
            'captcha_1': captcha_text,
        }

        response = self.client.post(
            reverse('kg_application_new'), form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix + reverse('index'))
        token = Application.objects.get().secret_token
        self.assertEqual(len(mail.outbox), 1)
        self.assertTrue(
            mail.outbox[0].subject.startswith('TestOrg invitation'))
        self.assertEqual(mail.outbox[0].from_email, settings.ACCOUNTS_EMAIL)
        self.assertEqual(mail.outbox[0].to[0], 'jim.bob@example.com')

        # SUBMIT APPLICANT DETAILS
        form_data = {
            'title': 'Mr',
            'short_name': 'Jim',
            'full_name': 'Jim Bob',
            'position': 'Researcher',
            'institute': 1,
            'department': 'Maths',
            'telephone': '4444444',
            'username': 'jimbob',
            'next': 'string',
        }

        response = self.client.post(
            reverse(
                'kg_application_unauthenticated',
                args=[token, 'O', 'applicant']),
            form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix + reverse(
                'kg_application_unauthenticated',
                args=[token, 'O', 'project']))

        # SUBMIT PROJECT DETAILS
        form_data = {
            'application_type': 'U',
            'project': 1,
            'aup': True,
            'make_leader': False,
            'additional_req': 'Meow',
            'needs_account': False,
            'submit': 'string',
        }

        response = self.client.post(
            reverse('kg_application_unauthenticated',
                    args=[token, 'O', 'project']),
            form_data, follow=True)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix
            + reverse('kg_application_unauthenticated', args=[token, 'L']))
        self.assertEqual(response.status_code, 200)
        applicant = Applicant.objects.get(username='jimbob')
        application = applicant.applications.all()[0]
        self.assertEqual(
            application.state,
            ProjectApplication.WAITING_FOR_LEADER)
        self.assertEqual(len(mail.outbox), 2)
        self.assertTrue(mail.outbox[1].subject.startswith('TestOrg request'))
        self.assertEqual(mail.outbox[1].from_email, settings.ACCOUNTS_EMAIL)
        self.assertEqual(mail.outbox[1].to[0], 'leader@example.com')

        # LEADER LOGS IN TO APPROVE
        logged_in = self.client.login(
            username='kgtestuser1', password='aq12ws')
        self.assertEqual(logged_in, True)

        # LEADER GET DETAILS
        response = self.client.get(
            reverse('kg_application_detail', args=[application.pk, 'L']))
        self.assertEqual(response.status_code, 200)

        # LEADER GET DECLINE PAGE
        response = self.client.get(
            reverse('kg_application_detail',
                    args=[application.pk, 'L', 'cancel']))
        self.assertEqual(response.status_code, 200)

        # LEADER GET APPROVE PAGE
        response = self.client.get(
            reverse('kg_application_detail',
                    args=[application.pk, 'L', 'approve']))
        self.assertEqual(response.status_code, 200)

        # LEADER APPROVE
        form_data = {
            'make_leader': False,
            'additional_req': 'Meow',
            'needs_account': False,
            'approve': True,
        }
        response = self.client.post(
            reverse('kg_application_detail',
                    args=[application.pk, 'L', 'approve']),
            form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix
            + reverse('kg_application_detail', args=[application.pk, 'K']))
        application = Application.objects.get(pk=application.id)
        self.assertEqual(application.state, Application.WAITING_FOR_ADMIN)
        self.assertEqual(len(mail.outbox), 3)
        self.client.logout()

        # ADMIN LOGS IN TO APPROVE
        set_admin()
        logged_in = self.client.login(username='kgsuper', password='aq12ws')
        self.assertEqual(logged_in, True)

        # ADMIN GET DETAILS
        response = self.client.get(
            reverse('kg_application_detail', args=[application.pk, 'K']))
        self.assertEqual(response.status_code, 200)

        # ADMIN GET DECLINE PAGE
        response = self.client.get(
            reverse('kg_application_detail',
                    args=[application.pk, 'K', 'cancel']))
        self.assertEqual(response.status_code, 200)

        # ADMIN GET APPROVE PAGE
        response = self.client.get(
            reverse('kg_application_detail',
                    args=[application.pk, 'K', 'approve']))
        self.assertEqual(response.status_code, 200)

        # ADMIN APPROVE
        form_data = {
            'make_leader': False,
            'additional_req': 'Woof',
            'needs_account': False,
            'approve': True,
        }
        response = self.client.post(
            reverse('kg_application_detail',
                    args=[application.pk, 'K', 'approve']),
            form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix
            + reverse('kg_application_detail', args=[application.pk, 'P']))
        application = Application.objects.get(pk=application.id)
        self.assertEqual(application.state, ProjectApplication.PASSWORD)
        self.assertEqual(len(mail.outbox), 4)
        self.client.logout()
        set_no_admin()

        # APPLICANT GET PASSWORD
        response = self.client.get(
            reverse('kg_application_unauthenticated', args=[token, 'P']))
        self.assertEqual(response.status_code, 200)

        # APPLICANT SET PASSWORD
        form_data = {
            'new_password1': "Exaiquouxei0",
            'new_password2': "Exaiquouxei0",
            'submit': 'string',
        }

        response = self.client.post(
            reverse('kg_application_unauthenticated',
                    args=[token, 'P']), form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix
            + reverse('kg_application_unauthenticated', args=[token, 'C']))

        # APPLICANT GET COMPLETE
        response = self.client.get(
            reverse('kg_application_unauthenticated', args=[token, 'C']))
        self.assertEqual(response.status_code, 200)

        # APPLICANT SET ARCHIVE
        form_data = {
            'archive': 'string',
        }
        response = self.client.post(
            reverse('kg_application_unauthenticated',
                    args=[token, 'C']),
            form_data, follow=False)
        # applicant not allowed to do this
        self.assertEqual(response.status_code, 400)
        self.client.logout()

        # ADMIN ARCHIVE
        set_admin()
        logged_in = self.client.login(username='kgsuper', password='aq12ws')
        self.assertEqual(logged_in, True)
        response = self.client.post(
            reverse('kg_application_detail',
                    args=[application.pk, 'C']),
            form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix
            + reverse('kg_application_detail', args=[application.pk, 'A']))
        self.client.logout()
        set_no_admin()

        # APPLICANT GET ARCHIVE
        response = self.client.get(
            reverse('kg_application_unauthenticated', args=[token, 'A']))
        self.assertEqual(response.status_code, 200)

        person = Person.objects.get(username='jimbob')
        self.assertEqual(person.title, 'Mr')
        self.assertEqual(person.short_name, 'Jim')
        self.assertEqual(person.full_name, 'Jim Bob')
        self.assertEqual(person.institute.name, 'Example')
        self.assertEqual(person.email, 'jim.bob@example.com')
        self.assertEqual(person.telephone, '4444444')
        self.assertEqual(person.approved_by.username, 'kgsuper')
        self.assertEqual(person.account_set.count(), 0)
        self.assertIsNotNone(person.date_approved)


@pytest.mark.django_db
class ProjectApplicationTestCase(TestCase):

    def setUp(self):
        call_command('loaddata', 'test_karaage', **{'verbosity': 0})

    def tearDown(self):
        set_admin()

    def test_register_account(self):
        if django.VERSION >= (1, 9):
            url_prefix = ""
        else:
            url_prefix = 'http://testserver'

        set_no_admin()

        with self.assertRaises(Person.DoesNotExist):
            Person.objects.get(username='jimbob')

        self.assertEqual(len(mail.outbox), 0)
        response = self.client.get(reverse('kg_application_new'))
        self.assertEqual(response.status_code, 200)
        a = response.content.find(
            b'name="captcha_0" value="') + 24
        b = a + 40
        hash_ = response.content[a:b].decode("ascii")

        captcha_text = CaptchaStore.objects.get(hashkey=hash_).response

        # OPEN APPLICATION
        form_data = {
            'email': 'jim.bob@example.com',
            'captcha_0': hash_,
            'captcha_1': captcha_text,
        }

        response = self.client.post(
            reverse('kg_application_new'), form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix + reverse('index'))
        token = Application.objects.get().secret_token
        self.assertEqual(len(mail.outbox), 1)
        self.assertTrue(
            mail.outbox[0].subject.startswith('TestOrg invitation'))
        self.assertEqual(mail.outbox[0].from_email, settings.ACCOUNTS_EMAIL)
        self.assertEqual(mail.outbox[0].to[0], 'jim.bob@example.com')

        # SUBMIT APPLICANT DETAILS
        form_data = {
            'title': 'Mr',
            'short_name': 'Jim',
            'full_name': 'Jim Bob',
            'position': 'Researcher',
            'institute': 1,
            'department': 'Maths',
            'telephone': '4444444',
            'username': 'jimbob',
            'next': 'string',
        }

        response = self.client.post(
            reverse('kg_application_unauthenticated',
                    args=[token, 'O', 'applicant']),
            form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix
            + reverse('kg_application_unauthenticated',
                      args=[token, 'O', 'project']))

        # SUBMIT PROJECT DETAILS
        form_data = {
            'application_type': 'P',
            'name': 'NewProject1',
            'description': "I like chocoloate.",
            'aup': True,
            'additional_req': 'Meow',
            'needs_account': False,
            'machine_categories': [1],
            'submit': 'string',
        }

        response = self.client.post(
            reverse('kg_application_unauthenticated',
                    args=[token, 'O', 'project']),
            form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix
            + reverse('kg_application_unauthenticated', args=[token, 'D']))
        applicant = Applicant.objects.get(username='jimbob')
        application = applicant.applications.all()[0]
        self.assertEqual(
            application.state, ProjectApplication.WAITING_FOR_DELEGATE)
        self.assertEqual(len(mail.outbox), 2)
        self.assertTrue(mail.outbox[1].subject.startswith('TestOrg request'))
        self.assertEqual(mail.outbox[1].from_email, settings.ACCOUNTS_EMAIL)
        self.assertEqual(mail.outbox[1].to[0], 'leader@example.com')

        # DELEGATE LOGS IN TO APPROVE
        logged_in = self.client.login(
            username='kgtestuser1', password='aq12ws')
        self.assertEqual(logged_in, True)

        # DELEGATE GET DETAILS
        response = self.client.get(
            reverse('kg_application_detail', args=[application.pk, 'D']))
        self.assertEqual(response.status_code, 200)

        # DELEGATE GET DECLINE PAGE
        response = self.client.get(
            reverse('kg_application_detail',
                    args=[application.pk, 'D', 'cancel']))
        self.assertEqual(response.status_code, 200)

        # DELEGATE GET APPROVE PAGE
        response = self.client.get(
            reverse('kg_application_detail',
                    args=[application.pk, 'D', 'approve']))
        self.assertEqual(response.status_code, 200)

        # DELEGATE APPROVE
        form_data = {
            'additional_req': 'Meow',
            'needs_account': False,
            'machine_categories': [1],
            'approve': True,
        }
        response = self.client.post(
            reverse('kg_application_detail',
                    args=[application.pk, 'D', 'approve']),
            form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix
            + reverse('kg_application_detail', args=[application.pk, 'K']))
        application = Application.objects.get(pk=application.id)
        self.assertEqual(application.state, Application.WAITING_FOR_ADMIN)
        self.assertEqual(len(mail.outbox), 3)
        self.client.logout()

        # ADMIN LOGS IN TO APPROVE
        set_admin()
        logged_in = self.client.login(username='kgsuper', password='aq12ws')
        self.assertEqual(logged_in, True)

        # ADMIN GET DETAILS
        response = self.client.get(
            reverse('kg_application_detail', args=[application.pk, 'K']))
        self.assertEqual(response.status_code, 200)

        # ADMIN GET DECLINE PAGE
        response = self.client.get(
            reverse('kg_application_detail',
                    args=[application.pk, 'K', 'cancel']))
        self.assertEqual(response.status_code, 200)

        # ADMIN GET APPROVE PAGE
        response = self.client.get(
            reverse('kg_application_detail',
                    args=[application.pk, 'K', 'approve']))
        self.assertEqual(response.status_code, 200)

        # ADMIN APPROVE
        form_data = {
            'additional_req': 'Woof',
            'needs_account': False,
            'machine_categories': [1],
            'approve': True,
        }
        response = self.client.post(
            reverse('kg_application_detail',
                    args=[application.pk, 'K', 'approve']),
            form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix
            + reverse('kg_application_detail', args=[application.pk, 'P']))
        application = Application.objects.get(pk=application.id)
        self.assertEqual(application.state, ProjectApplication.PASSWORD)
        self.assertEqual(len(mail.outbox), 4)
        self.client.logout()
        set_no_admin()

        # APPLICANT GET PASSWORD
        response = self.client.get(
            reverse('kg_application_unauthenticated', args=[token, 'P']))
        self.assertEqual(response.status_code, 200)

        # APPLICANT SET PASSWORD
        form_data = {
            'new_password1': "Exaiquouxei0",
            'new_password2': "Exaiquouxei0",
            'submit': 'string',
        }

        response = self.client.post(
            reverse('kg_application_unauthenticated', args=[token, 'P']),
            form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix
            + reverse('kg_application_unauthenticated', args=[token, 'C']))

        # APPLICANT GET COMPLETE
        response = self.client.get(
            reverse('kg_application_unauthenticated', args=[token, 'C']))
        self.assertEqual(response.status_code, 200)

        # APPLICANT SET ARCHIVE
        form_data = {
            'archive': 'string',
        }
        response = self.client.post(
            reverse('kg_application_unauthenticated', args=[token, 'C']),
            form_data, follow=False)
        # applicant not allowed to do this
        self.assertEqual(response.status_code, 400)
        self.client.logout()

        # ADMIN ARCHIVE
        set_admin()
        logged_in = self.client.login(username='kgsuper', password='aq12ws')
        self.assertEqual(logged_in, True)
        response = self.client.post(
            reverse('kg_application_detail', args=[application.pk, 'C']),
            form_data, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(
            response.redirect_chain[0][0],
            url_prefix
            + reverse('kg_application_detail', args=[application.pk, 'A']))
        self.client.logout()
        set_no_admin()

        # APPLICANT GET ARCHIVE
        response = self.client.get(
            reverse('kg_application_unauthenticated', args=[token, 'A']))
        self.assertEqual(response.status_code, 200)

        person = Person.objects.get(username='jimbob')
        self.assertEqual(person.title, 'Mr')
        self.assertEqual(person.short_name, 'Jim')
        self.assertEqual(person.full_name, 'Jim Bob')
        self.assertEqual(person.institute.name, 'Example')
        self.assertEqual(person.telephone, '4444444')
        self.assertEqual(person.approved_by.username, 'kgsuper')
        self.assertEqual(person.account_set.count(), 0)
        self.assertIsNotNone(person.date_approved)

    def _test_project_approval(self, application):
        admin = fixtures.PersonFactory(username='admin')
        admin.is_admin = True
        admin.save()
        application.approve(admin)

    def _test_project_make_leader(self, application):
        self._test_project_approval(application)
        if application.make_leader:
            self.assertIn(application.applicant,
                          application.project.leaders.all())
        else:
            self.assertNotIn(application.applicant,
                             application.project.leaders.all())

    def test_new_project_make_leader(self, make_leader=True):
        application = ExistingProjectApplicationFactory()
        application.make_leader = make_leader
        self._test_project_make_leader(application)

    def test_new_project_make_leader_false(self):
        self.test_new_project_make_leader(make_leader=False)

    def test_existing_project_make_leader(self, make_leader=True):
        application = NewProjectApplicationFactory()
        application.make_leader = make_leader
        self._test_project_make_leader(application)

    def test_existing_project_make_leader_false(self):
        self.test_existing_project_make_leader(make_leader=False)



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import sys

from karaage.conf.defaults import *  # NOQA
from karaage.conf.process import post_process
from karaage.tests.defaults import *  # NOQA


PLUGINS = [
    'karaage.plugins.kgsoftware.plugin',
    'karaage.plugins.kgsoftware.applications.plugin',
]
DEBUG = False
ALLOWED_HOSTS = ["localhost"]

post_process(sys.modules[__name__])



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import datetime
import warnings

import six
from django.contrib.auth.models import AbstractBaseUser
from django.db import models
from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible
from jsonfield import JSONField
from model_utils import FieldTracker

from karaage.common import is_admin, log
from karaage.common.constants import COUNTRIES, STATES, TITLES
from karaage.people.managers import (
    ActivePersonManager,
    DeletedPersonManager,
    LeaderManager,
    PersonManager,
)


# Note on terminology:
#
# A inactive person is a person who has been deleted. We keep there entry
# around and don't actually delete it.
#
# A locked person is a person who has not been deleted but is not allowed
# access for some reason.

@python_2_unicode_compatible
class Person(AbstractBaseUser):
    username = models.CharField(max_length=255, unique=True)
    email = models.EmailField(null=True, db_index=True)
    short_name = models.CharField(max_length=30)
    full_name = models.CharField(max_length=60)
    is_active = models.BooleanField(default=True)
    is_admin = models.BooleanField(default=False)

    saml_id = models.CharField(
        max_length=200, null=True, blank=True, unique=True, editable=False)
    position = models.CharField(max_length=200, null=True, blank=True)
    telephone = models.CharField(max_length=200, null=True, blank=True)
    mobile = models.CharField(max_length=200, null=True, blank=True)
    department = models.CharField(max_length=200, null=True, blank=True)
    supervisor = models.CharField(max_length=100, null=True, blank=True)
    institute = models.ForeignKey('karaage.Institute', on_delete=models.CASCADE)
    title = models.CharField(
        choices=TITLES, max_length=10, null=True, blank=True)
    address = models.CharField(max_length=200, null=True, blank=True)
    city = models.CharField(max_length=100, null=True, blank=True)
    postcode = models.CharField(max_length=8, null=True, blank=True)
    state = models.CharField(
        choices=STATES, max_length=4, null=True, blank=True)
    country = models.CharField(
        max_length=2, choices=COUNTRIES, null=True, blank=True)
    website = models.URLField(null=True, blank=True)
    fax = models.CharField(max_length=50, null=True, blank=True)
    comment = models.TextField(null=True, blank=True)
    approved_by = models.ForeignKey(
        'self', related_name='user_approver', null=True, blank=True,
        on_delete=models.SET_NULL)
    deleted_by = models.ForeignKey(
        'self', related_name='user_deletor', null=True, blank=True,
        on_delete=models.SET_NULL)
    date_approved = models.DateField(null=True, blank=True)
    date_deleted = models.DateField(null=True, blank=True)
    last_usage = models.DateField(null=True, blank=True)
    expires = models.DateField(null=True, blank=True)
    is_systemuser = models.BooleanField(default=False)
    login_enabled = models.BooleanField(default=True)
    legacy_ldap_password = models.CharField(
        max_length=128, null=True, blank=True)
    objects = PersonManager()
    active = ActivePersonManager()
    deleted = DeletedPersonManager()
    projectleaders = LeaderManager()

    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = ['email', 'short_name', 'full_name', 'institute']

    _tracker = FieldTracker()

    def __init__(self, *args, **kwargs):
        super(Person, self).__init__(*args, **kwargs)
        self._raw_password = None

    class Meta:
        verbose_name_plural = 'people'
        ordering = ['full_name', 'short_name']
        db_table = 'person'
        app_label = 'karaage'

    def __str__(self):
        name = self.get_full_name()
        if not name:
            name = "No Name"
        return name

    def get_absolute_url(self):
        return reverse('kg_person_detail', kwargs={'username': self.username})

    def save(self, *args, **kwargs):
        created = self.pk is None

        # save the object
        super(Person, self).save(*args, **kwargs)

        if created:
            log.add(self, 'Created')
        for field in self._tracker.changed():
            if field != "password":
                log.change(
                    self,
                    'Changed %s to %s' % (field, getattr(self, field)))

        # update datastores
        from karaage.datastores import save_account
        for ua in self.account_set.filter(date_deleted__isnull=True):
            save_account(ua)

        # has locked status changed?
        if self._tracker.has_changed("login_enabled"):
            if self.login_enabled:
                for ua in self.account_set.filter(date_deleted__isnull=True):
                    ua.unlock()
            else:
                for ua in self.account_set.filter(date_deleted__isnull=True):
                    ua.lock()

        # has the institute changed?
        if self._tracker.has_changed("institute_id"):
            from karaage.institutes.models import Institute
            old_institute_pk = self._tracker.previous("institute_id")
            new_institute = self.institute
            if old_institute_pk is not None:
                old_institute = Institute.objects.get(pk=old_institute_pk)
                from karaage.datastores import remove_accounts_from_institute
                query = self.account_set
                remove_accounts_from_institute(query, old_institute)
            if new_institute is not None:
                from karaage.datastores import add_accounts_to_institute
                query = self.account_set
                add_accounts_to_institute(query, new_institute)

        if self._raw_password is not None:
            for ua in self.account_set.filter(date_deleted__isnull=True):
                ua.set_password(self._raw_password)
                ua.save()
            log.change(self, 'Changed Password')
            self._raw_password = None
    save.alters_data = True

    def delete(self, *args, **kwargs):
        # delete the object
        log.delete(self, 'Deleted')
        super(Person, self).delete(*args, **kwargs)
    delete.alters_data = True

    @property
    def last_name(self):
        if not self.full_name:
            return None
        elif self.full_name.find(" ") != -1:
            _, _, last_name = self.full_name.rpartition(" ")
            return last_name.strip()
        else:
            return self.full_name

    @property
    def first_name(self):
        if not self.full_name:
            return None
        elif self.full_name.find(" ") != -1:
            first_name, _, _ = self.full_name.rpartition(" ")
            return first_name.strip()
        else:
            return self.full_name

    def has_perm(self, perm, obj=None):
        """Does the user have a specific permission? (depreciated)"""
        warnings.warn('Person.has_perm obsolete (get)', DeprecationWarning)
        raise RuntimeError("Do not use")

    def has_module_perms(self, app_label):
        """(depreciated)"""
        warnings.warn(
            'Person.has_module_perms obsolete (get)', DeprecationWarning)
        raise RuntimeError("Do not use")

    @property
    def user(self):
        """(depreciated)"""
        warnings.warn(
            'Person.user obsolete (get)', DeprecationWarning)
        raise RuntimeError("Do not use")

    def get_profile(self):
        """(depreciated)"""
        warnings.warn(
            'Person.get_profile() obsolete (get)', DeprecationWarning)
        raise RuntimeError("Do not use")

    @property
    def is_staff(self):
        """Is the user a member of staff? (depreciated)"""
        warnings.warn(
            'Person.has_module_perms obsolete (get)', DeprecationWarning)
        raise RuntimeError("Do not use")

    # Can person view this self record?
    def can_view(self, request):
        from karaage.projects.models import Project
        person = request.user

        if not person.is_authenticated:
            return False

        # ensure person making request isn't deleted.
        if not person.is_active:
            return False

        # ensure person making request isn't locked.
        if person.is_locked():
            return False

        # staff members can view everything
        if is_admin(request):
            return True

        # we don't allow people to see inactive accounts.
        if not self.is_active:
            return False

        # person can view own self
        if self.id == person.id:
            return True

        # Institute delegate==person can view any member of institute
        if self.institute.is_active:
            if person in self.institute.delegates.all():
                return True

        # Institute delegate==person can view people in projects that are a
        # member of institute
        if Project.objects.filter(group__members=self.id) \
                .filter(institute__delegates=person):
            return True

        # person can view people in projects they belong to
        tmp = Project.objects.filter(group__members=self.id) \
            .filter(group__members=person.id) \
            .filter(is_active=True)
        if tmp.count() > 0:
            return True

        # Leader==person can view people in projects they lead
        tmp = Project.objects.filter(group__members=self.id) \
            .filter(leaders=person.id) \
            .filter(is_active=True)
        if tmp.count() > 0:
            return True
        return False

    def get_full_name(self):
        """ Get the full name of the person. """
        return self.full_name

    def get_short_name(self):
        """ Get the abbreviated name of the person. """
        return self.short_name

    def has_account(self):
        ua = self.account_set.all()
        ua = ua.filter(date_deleted__isnull=True)
        if ua.count() != 0:
            return True
        return False

    def get_account(self):
        try:
            return self.account_set.get(date_deleted__isnull=True)
        except self.account_set.DoesNotExist:
            return None
        except self.account_set.MultipleObjectsReturned:
            return None

    def is_leader(self):
        if self.leads.count() > 0:
            return True
        return False

    def activate(self, approved_by):
        if not self.is_active:
            self.date_approved = datetime.datetime.today()

            self.approved_by = approved_by
            self.deleted_by = None
            self.date_deleted = None
            self.is_active = True
            self.save()

            log.change(self, 'Activated by %s' % approved_by)
    activate.alters_data = True

    def deactivate(self, deleted_by):
        """ Sets Person not active and deletes all Accounts"""
        # deactivate accounts first, so we don't need to repeatedly update
        # their datastores uselessly when clearing the groups.
        for ua in self.account_set.filter(date_deleted__isnull=True):
            ua.deactivate()

        # now we can clear the groups
        self.groups.clear()

        # finally, mark the person as deleted.
        self.is_active = False
        self.expires = None
        self.date_deleted = datetime.datetime.today()
        self.deleted_by = deleted_by
        self.save()

        # log a message
        log.change(self, 'Deactivated by %s' % deleted_by)
    deactivate.alters_data = True

    def set_password(self, password):
        super(Person, self).set_password(password)
        if self.legacy_ldap_password is not None:
            self.legacy_ldap_password = None
        self._raw_password = password
    set_password.alters_data = True

    def lock(self):
        if self.is_locked():
            return
        self.login_enabled = False
        self.save()
    lock.alters_data = True

    def unlock(self):
        if not self.is_locked():
            return
        self.login_enabled = True
        self.save()
    unlock.alters_data = True

    def is_locked(self):
        return not self.login_enabled

    def add_group(self, group):
        group.members.add(self)
    add_group.alters_data = True

    def remove_group(self, group):
        group.members.remove(self)
    remove_group.alters_data = True

    @property
    def projects(self):
        from karaage.projects.models import Project
        return Project.objects.filter(group__members=self)

    @property
    def institutes(self):
        from karaage.institutes.models import Institute
        return Institute.objects.filter(group__members=self)


@python_2_unicode_compatible
class Group(models.Model):

    """Groups represent collections of people, these objects can be
    expressed externally in a datastore."""
    name = models.CharField(max_length=255, unique=True)
    foreign_id = models.CharField(
        max_length=255, null=True, unique=True,
        help_text='The foreign identifier from the datastore.')
    members = models.ManyToManyField(Person, related_name='groups')
    description = models.TextField(null=True, blank=True)
    extra_data = JSONField(
        default={},
        help_text='Datastore specific values should be stored in this field.')

    _tracker = FieldTracker()

    class Meta:
        ordering = ['name']
        db_table = 'people_group'
        app_label = 'karaage'

    def __str__(self):
        return six.u("%s") % self.name

    def get_absolute_url(self):
        return reverse('kg_group_detail', kwargs={'group_name': self.name})

    def save(self, *args, **kwargs):
        created = self.pk is None

        # save the object
        super(Group, self).save(*args, **kwargs)

        if created:
            log.add(self, 'Created')
        for field in self._tracker.changed():
            log.field_change(self, field=field, new_value=getattr(self, field))

        old_name = self._tracker.previous("name")
        new_name = self.name
        if old_name is not None and old_name != new_name:
            from karaage.datastores import set_group_name
            set_group_name(self, old_name, new_name)
            log.change(self, "Renamed group")

        # update the datastore
        from karaage.datastores import save_group
        save_group(self)

    save.alters_data = True

    def delete(self, *args, **kwargs):
        for person in self.members.all():
            _remove_person_from_group(person, self)

        # delete the object
        log.delete(self, 'Deleted')
        super(Group, self).delete(*args, **kwargs)

        # update the datastore
        from karaage.datastores import delete_group
        delete_group(self)
    delete.alters_data = True

    def add_person(self, person):
        self.members.add(person)
    add_person.alters_data = True

    def remove_person(self, person):
        self.members.remove(person)
    remove_person.alters_data = True


def _add_person_to_group(person, group):
    """ Call datastores after adding a person to a group. """
    from karaage.datastores import add_accounts_to_group
    from karaage.datastores import add_accounts_to_project
    from karaage.datastores import add_accounts_to_institute

    a_list = person.account_set
    add_accounts_to_group(a_list, group)
    for project in group.project_set.all():
        add_accounts_to_project(a_list, project)
    for institute in group.institute_set.all():
        add_accounts_to_institute(a_list, institute)


def _remove_person_from_group(person, group):
    """ Call datastores after removing a person from a group. """
    from karaage.datastores import remove_accounts_from_group
    from karaage.datastores import remove_accounts_from_project
    from karaage.datastores import remove_accounts_from_institute

    a_list = person.account_set
    remove_accounts_from_group(a_list, group)
    for project in group.project_set.all():
        remove_accounts_from_project(a_list, project)
    for institute in group.institute_set.all():
        remove_accounts_from_institute(a_list, institute)


def _members_changed(
        sender, instance, action, reverse, model, pk_set, **kwargs):
    """
    Hook that executes whenever the group members are changed.
    """
    if action == "post_add":
        if not reverse:
            group = instance
            for person in model.objects.filter(pk__in=pk_set):
                log.change(person, "Added person to group %s" % group)
                log.change(group, "Added person %s to group" % person)
                _add_person_to_group(person, group)
        else:
            person = instance
            for group in model.objects.filter(pk__in=pk_set):
                log.change(person, "Added person to group %s" % group)
                log.change(group, "Added person %s to group" % person)
                _add_person_to_group(person, group)

    elif action == "post_remove":
        if not reverse:
            group = instance
            for person in model.objects.filter(pk__in=pk_set):
                log.change(person, "Removed person from group %s" % group)
                log.change(group, "Removed person %s from group" % person)
                _remove_person_from_group(person, group)
        else:
            person = instance
            for group in model.objects.filter(pk__in=pk_set):
                log.change(person, "Removed person from group %s" % group)
                log.change(group, "Removed person %s from group" % person)
                _remove_person_from_group(person, group)

    elif action == "pre_clear":
        # This has to occur in pre_clear, not post_clear, as otherwise
        # we won't see what groups need to be removed.
        if not reverse:
            group = instance
            log.change(group, "Removed all people from group")
            for person in group.members.all():
                log.change(group, "Removed person %s from group" % person)
                _remove_person_from_group(person, group)
        else:
            person = instance
            log.change(person, "Removed person from all groups")
            for group in person.groups.all():
                log.change(group, "Removed person %s from group" % person)
                _remove_person_from_group(person, group)


models.signals.m2m_changed.connect(
    _members_changed, sender=Group.members.through)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-03-02 18:08
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('kgusage', '0003_auto_20180131_1758'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cpujob',
            name='account',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='karaage.Account'),
        ),
        migrations.AlterField(
            model_name='cpujob',
            name='machine',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='karaage.Machine'),
        ),
        migrations.AlterField(
            model_name='cpujob',
            name='project',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='karaage.Project'),
        ),
        migrations.AlterField(
            model_name='cpujob',
            name='queue',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='kgusage.Queue'),
        ),
    ]



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

try:
    # Python 3
    import http.client as httplib
except ImportError:
    # Python 2
    import httplib

try:
    # Python 3
    import xmlrpc.client as xmlrpclib
except ImportError:
    # Python 2
    import xmlrpclib

import pytest
from django.test import TestCase

from karaage.machines.models import Account
from karaage.people.models import Group, Person


class DjangoTestClientTransport(object):
    client = None

    def __init__(self, client):
        self.client = client

    def getparser(self):
        return xmlrpclib.getparser()

    def request(self, host, handler, request_body, verbose=False):
        parser, unmarshaller = self.getparser()
        response = self.client.post(handler, request_body, 'text/xml')
        if response.status_code != 200:
            raise xmlrpclib.ProtocolError(
                '%s%s' % (host, handler),
                response.status_code,
                httplib.responses.get(response.status_code, ''),
                dict(response.items()),
            )
        parser.feed(response.content)
        return unmarshaller.close()


@pytest.mark.django_db
class XmlrpcTestCase(TestCase):
    fixtures = [
        'test_karaage.json',
    ]

    def get_server_proxy(self):
        return xmlrpclib.ServerProxy(
            'http://testserver/xmlrpc/',
            transport=DjangoTestClientTransport(self.client),
        )

    def setUp(self):
        super(XmlrpcTestCase, self).setUp()
        self.server = self.get_server_proxy()

    def test_get_disk_quota(self):
        server = self.server

        result = server.get_disk_quota("kgtestuser1")
        self.assertEqual(result, "Account not found")

        result = server.get_disk_quota("kgtestuser3")
        self.assertEqual(result, False)
        result = server.get_disk_quota("kgtestuser3", "tango")
        self.assertEqual(result, False)

        account = Account.objects.get(username="kgtestuser3")
        account.disk_quota = 1
        account.save()

        result = server.get_disk_quota("kgtestuser3")
        self.assertEqual(result, 1048576)
        result = server.get_disk_quota("kgtestuser3", "tango")
        self.assertEqual(result, 1048576)

    def test_get_projects(self):
        server = self.server

        with self.assertRaises(xmlrpclib.Fault) as cm:
            server.get_projects("tango", "aqws12")
        self.assertEqual(cm.exception.faultCode, 81)
        self.assertEqual(
            cm.exception.faultString, 'Username and/or password is incorrect')

        result = server.get_projects("tango", "aq12ws")
        self.assertEqual(result, ['TestProject1'])

        result = server.get_projects("wexstan", "aq12ws")
        self.assertEqual(result, ['TestProject1'])

        result = server.get_projects("edda", "aq12ws")
        self.assertEqual(result, ['TestProject1'])

    def test_get_project(self):
        server = self.server

        # account does not exist
        result = server.get_project("kgtestuser1", "TestProject1")
        self.assertEqual(result, "Account 'kgtestuser1' not found")

        # project does exist, and person belongs to it
        result = server.get_project("kgtestuser3", "TestProject1")
        self.assertEqual(result, "TestProject1")

        result = server.get_project("kgtestuser3", "TestProject1", "tango")
        self.assertEqual(result, "TestProject1")

        result = server.get_project("kgtestuser3", "TestProject1", "wexstan")
        self.assertEqual(result, "TestProject1")

        result = server.get_project("kgtestuser3", "TestProject1", "edda")
        self.assertEqual(result, "TestProject1")

        # project does not exist - should fall back to default
        result = server.get_project("kgtestuser3", "TestProjectx", "tango")
        self.assertEqual(result, "TestProject1")

        result = server.get_project("kgtestuser3", "TestProjectx", "wexstan")
        self.assertEqual(result, "TestProject1")

        result = server.get_project("kgtestuser3", "TestProjectx", "edda")
        self.assertEqual(result, "TestProject1")

        # project does exist, and person doesn't belong to it
        # in this case default fall back fails too
        person = Person.objects.get(username="kgtestuser3")
        group = Group.objects.get(name="testproject1")
        group.members.remove(person)

        result = server.get_project("kgtestuser3", "TestProject1")
        self.assertEqual(result, "None")

        result = server.get_project("kgtestuser3", "TestProject1", "tango")
        self.assertEqual(result, "None")

        result = server.get_project("kgtestuser3", "TestProject1", "wexstan")
        self.assertEqual(result, "None")

        result = server.get_project("kgtestuser3", "TestProject1", "edda")
        self.assertEqual(result, "None")

    def test_get_project_members(self):
        server = self.server

        with self.assertRaises(xmlrpclib.Fault) as cm:
            server.get_project_members("tango", "aqws12", "TestProject2")
        self.assertEqual(cm.exception.faultCode, 81)
        self.assertEqual(
            cm.exception.faultString, 'Username and/or password is incorrect')

        # Project has no ProjectQuota
        result = server.get_project_members("tango", "aq12ws", "TestProject2")
        self.assertEqual(result, "Project not found")

        result = server.get_project_members(
            "wexstan", "aq12ws", "TestProject2")
        self.assertEqual(result, "Project not found")

        result = server.get_project_members("edda", "aq12ws", "TestProject2")
        self.assertEqual(result, "Project not found")

        # Project has ProjectQuota for default machine category
        result = server.get_project_members("tango", "aq12ws", "TestProject1")
        self.assertEqual(result, ['kgtestuser3'])

        result = server.get_project_members(
            "wexstan", "aq12ws", "TestProject1")
        self.assertEqual(result, ['kgtestuser3'])

        result = server.get_project_members("edda", "aq12ws", "TestProject1")
        self.assertEqual(result, ['kgtestuser3'])

    def test_get_users_project(self):
        server = self.server

        with self.assertRaises(xmlrpclib.Fault) as cm:
            server.get_users_projects("tango", "aq12ws")
        self.assertEqual(cm.exception.faultCode, 81)
        self.assertEqual(
            cm.exception.faultString, 'Username and/or password is incorrect')

        result = server.get_users_projects("kgtestuser1", "aq12ws")
        self.assertEqual(result, [0, []])

        result = server.get_users_projects("kgtestuser2", "aq12ws")
        self.assertEqual(result, [0, []])

        result = server.get_users_projects("kgtestuser3", "aq12ws")
        self.assertEqual(result, [0, ['TestProject1']])



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

""" Application specific tags. """

import django_tables2 as tables
from django import template

from karaage.people.tables import PersonTable

from ..views.base import get_state_machine


register = template.Library()


@register.simple_tag(takes_context=True)
def application_state(context, application):
    """ Render current state of application, verbose. """
    new_context = {
        'roles': context['roles'],
        'org_name': context['org_name'],
        'application': application,
    }
    nodelist = template.loader.get_template(
        'kgapplications/%s_common_state.html' % application.type)
    output = nodelist.render(new_context)
    return output


@register.simple_tag(takes_context=True)
def application_request(context, application):
    """ Render current detail of application, verbose. """
    new_context = {
        'roles': context['roles'],
        'org_name': context['org_name'],
        'application': application,
    }
    nodelist = template.loader.get_template(
        'kgapplications/%s_common_request.html' % application.type)
    output = nodelist.render(new_context)
    return output


@register.simple_tag(takes_context=True)
def application_simple_state(context, application):
    """ Render current state of application, verbose. """
    state_machine = get_state_machine(application)
    state = state_machine.get_state(application)
    return state.name


@register.inclusion_tag(
    'kgapplications/common_actions.html', takes_context=True)
def application_actions(context):
    """ Render actions available. """
    return {
        'roles': context['roles'],
        'actions': context['actions'],
        'extra': "",
    }


@register.tag(name="application_actions_plus")
def do_application_actions_plus(parser, token):
    """ Render actions available with extra text. """
    nodelist = parser.parse(('end_application_actions',))
    parser.delete_first_token()
    return ApplicationActionsPlus(nodelist)


class ApplicationActionsPlus(template.Node):
    """ Node for rendering actions available with extra text. """

    def __init__(self, nodelist):
        super(ApplicationActionsPlus, self).__init__()
        self.nodelist = nodelist

    def render(self, context):
        extra = self.nodelist.render(context)
        nodelist = template.loader.get_template(
            'kgapplications/common_actions.html')
        new_context = {
            'roles': context['roles'],
            'extra': extra,
            'actions': context['actions'],
        }
        output = nodelist.render(new_context)
        return output


@register.simple_tag(takes_context=True)
def get_similar_people_table(context, applicant):
    queryset = applicant.similar_people()
    table = PersonTable(
        queryset,
        empty_text="(No potential duplicates found, please check manually)")
    config = tables.RequestConfig(context['request'], paginate={"per_page": 5})
    config.configure(table)
    return table



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.
import datetime
import os.path
from decimal import Decimal, DivisionByZero, InvalidOperation

import django
from django import template
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.http import QueryDict
from django.utils.html import escape
from django.utils.safestring import mark_safe

from karaage.common.models import COMMENT, LogEntry
from karaage.plugins import BasePlugin


register = template.Library()


class UrlWithParamNode(template.Node):

    def __init__(self, copy, nopage, changes):
        self.copy = copy
        self.nopage = nopage
        self.changes = []
        for key, newvalue in changes:
            newvalue = template.Variable(newvalue)
            self.changes.append((key, newvalue,))

    def render(self, context):
        if 'request' not in context:
            return ""

        request = context['request']

        result = {}

        if self.copy:
            result = request.GET.copy()
        else:
            result = QueryDict("", mutable=True)

        if self.nopage:
            result.pop("page", None)

        for key, newvalue in self.changes:
            newvalue = newvalue.resolve(context)
            result[key] = newvalue

        return "?" + result.urlencode()


@register.tag
def url_with_param(parser, token):
    bits = token.split_contents()
    qschanges = []

    bits.pop(0)

    copy = False
    if bits[0] == "copy":
        copy = True
        bits.pop(0)

    nopage = False
    if bits[0] == "nopage":
        nopage = True
        bits.pop(0)

    for i in bits:
        try:
            key, newvalue = i.split('=', 1)
            qschanges.append((key, newvalue,))
        except ValueError:
            raise template.TemplateSyntaxError(
                "Argument syntax wrong: should be key=value")
    return UrlWithParamNode(copy, nopage, qschanges)


@register.inclusion_tag('karaage/common/comments.html', takes_context=True)
def comments(context, obj):
    """ Render comments for obj. """
    content_type = ContentType.objects.get_for_model(obj.__class__)
    comment_list = LogEntry.objects.filter(
        content_type=content_type,
        object_id=obj.pk,
        action_flag=COMMENT
    )
    return {
        'obj': obj,
        'comment_list': comment_list,
        'is_admin': context['is_admin'],
    }


@register.simple_tag
def comment_count(obj):
    content_type = ContentType.objects.get_for_model(obj.__class__)
    comment_list = LogEntry.objects.filter(
        content_type=content_type,
        object_id=obj.pk,
        action_flag=COMMENT
    )
    return int(comment_list.count())


@register.simple_tag
def active(request, pattern):
    import re
    spec = '^%s/%s' % (request.META['SCRIPT_NAME'], pattern)
    if re.search(spec, request.path):
        return 'active'
    return ''


@register.simple_tag
def date_filter(start, end):
    result = QueryDict("", mutable=True)

    today = datetime.date.today()

    last_7 = (today - datetime.timedelta(days=7)).strftime('%Y-%m-%d')
    last_90 = (today - datetime.timedelta(days=90)).strftime('%Y-%m-%d')
    last_365 = (today - datetime.timedelta(days=365)).strftime('%Y-%m-%d')

    view_7, view_90, view_365 = False, False, False

    if end == today:
        if start == today - datetime.timedelta(days=7):
            view_7 = True
        if start == today - datetime.timedelta(days=90):
            view_90 = True
        if start == today - datetime.timedelta(days=365):
            view_365 = True

    s = []

    if view_7:
        s.append('Last 7 Days')
    else:
        result["start"] = last_7
        url = ".?" + result.urlencode()
        s.append("""<a href="%s">Last 7 Days</a>""" % escape(url))

    if view_90:
        s.append("Last 90 Days")
    else:
        result["start"] = last_90
        url = ".?" + result.urlencode()
        s.append("""<a href="%s">Last 90 Days</a>""" % escape(url))

    if view_365:
        s.append("Last 365 Days")
    else:
        result["start"] = last_365
        url = ".?" + result.urlencode()
        s.append("""<a href="%s">Last 365 Days</a>""" % escape(url))

    return mark_safe(" | ".join(s))


@register.simple_tag
def yes_no(boolean, true_msg='Yes', false_msg='No'):
    if reversed == 'reversed':
        if boolean:
            boolean = False
        else:
            boolean = True

    if boolean:
        return mark_safe("<span class='yes'>%s</span>" % escape(true_msg))
    else:
        return mark_safe("<span class='no'>%s</span>" % escape(false_msg))


class SearchFormNode(template.Node):

    def __init__(self, post_url):
        self.post_url = post_url

    def render(self, context):
        template_obj = template.loader.get_template(
            'karaage/common/search_form.html')
        context.push()
        context['post_url'] = self.post_url
        output = template_obj.render(context)
        context.pop()
        return output


@register.simple_tag
def divide(a, b):
    two_places = Decimal(10) ** -2
    try:
        return (Decimal(a) / Decimal(b) * 100).quantize(two_places)
    except (DivisionByZero, InvalidOperation):
        return ''


def get_app_labels():
    if django.VERSION < (1, 7):
        for app in settings.INSTALLED_APPS:
            _, _, label = app.rpartition(".")
            if label is not None:
                yield label
    else:
        from django.apps import apps
        for config in apps.get_app_configs():
            if isinstance(config, BasePlugin):
                yield config.label


class ForEachAppIncludeNode(template.Node):

    def __init__(self, template_name):
        self.template_name = template.Variable(template_name)

    def render(self, context):
        template_name = self.template_name.resolve(context)

        result = []
        for label in get_app_labels():
            template_path = os.path.join(label, template_name)
            try:
                template_obj = template.loader.get_template(template_path)
            except template.TemplateDoesNotExist:
                pass
            else:
                context.push()
                output = template_obj.render(context.flatten())
                result.append(output)
                context.pop()

        return "".join(result)


@register.tag
def for_each_app_include(parser, token):
    try:
        tag_name, template_name = token.split_contents()
    except ValueError:
        raise template.TemplateSyntaxError(
            "%r tag requires one arguments" % token.contents.split()[0])
    return ForEachAppIncludeNode(template_name)


@register.simple_tag()
def is_for_each_app_include_empty(template_name):
    for label in get_app_labels():
        template_path = os.path.join(label, template_name)
        try:
            template.loader.get_template(template_path)
            return False
        except template.TemplateDoesNotExist:
            pass
    return True



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import, unicode_literals

import six
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.encoding import python_2_unicode_compatible, smart_text
from django.utils.translation import ugettext
from django.utils.translation import ugettext_lazy as _

from karaage.middleware.threadlocals import get_current_user


try:
    # Django >= 1.7
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    # Django < 1.7
    from django.contrib.contenttypes.generic import GenericForeignKey


ADDITION = 1
CHANGE = 2
DELETION = 3
COMMENT = 4


class LogEntryManager(models.Manager):

    def log_action(self, user_id, content_type_id, object_id,
                   object_repr, action_flag, change_message=''):
        msg = self.model(None, None, user_id, content_type_id, object_id,
                         object_repr[:200], action_flag, change_message)
        msg.save()
        return msg

    def log_object(self, obj, flag, message, user=None):
        assert obj is not None
        assert obj.pk is not None
        if user is None:
            user = get_current_user()
        if user is None:
            user_id = None
        else:
            user_id = user.pk
        return self.log_action(
            user_id=user_id,
            content_type_id=ContentType.objects.get_for_model(obj).pk,
            object_id=obj.pk,
            object_repr=six.text_type(obj),
            action_flag=flag,
            change_message=message)


@python_2_unicode_compatible
class LogEntry(models.Model):
    action_time = models.DateTimeField(_('action time'), auto_now_add=True)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, null=True, on_delete=models.SET_NULL)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.TextField(_('object id'), blank=True, null=True)
    content_object = GenericForeignKey('content_type', 'object_id')
    object_repr = models.CharField(_('object repr'), max_length=200)
    action_flag = models.PositiveSmallIntegerField(_('action flag'))
    change_message = models.TextField(_('change message'), blank=True)

    objects = LogEntryManager()

    class Meta:
        verbose_name = _('log entry')
        verbose_name_plural = _('log entries')
        db_table = 'admin_log'
        app_label = 'karaage'
        ordering = ('-action_time', '-pk')

    def __repr__(self):
        return smart_text(self.action_time)

    def __str__(self):
        if self.action_flag == ADDITION:
            return ugettext('Added "%(object)s".') % \
                {'object': self.object_repr}
        elif self.action_flag == CHANGE:
            return ugettext('Changed "%(object)s" - %(changes)s') % {
                'object': self.object_repr,
                'changes': self.change_message,
            }
        elif self.action_flag == DELETION:
            return ugettext('Deleted "%(object)s."') % \
                {'object': self.object_repr}
        elif self.action_flag == COMMENT:
            return ugettext('Comment "%(object)s" - %(changes)s') % {
                'object': self.object_repr,
                'changes': self.change_message,
            }

        return ugettext('LogEntry Object')

    def is_addition(self):
        return self.action_flag == ADDITION

    def is_change(self):
        return self.action_flag == CHANGE

    def is_deletion(self):
        return self.action_flag == DELETION

    def is_comment(self):
        return self.action_flag == COMMENT



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

try:
    # Python 3
    import xmlrpc.client as xmlrpclib
except ImportError:
    # Python 2
    import xmlrpclib

import datetime
import json
import os.path

import alogger.tests.examples
import pytest
from alogger import get_parser
from django.test import TestCase

from karaage.institutes.models import Institute
from karaage.machines.models import Account
from karaage.people.models import Person
from karaage.projects.models import Project
from karaage.tests.test_xmlrpc import DjangoTestClientTransport


@pytest.mark.django_db
class XmlrpcTestCase(TestCase):
    fixtures = [
        'test_karaage.json',
    ]

    def setUp(self):
        super(XmlrpcTestCase, self).setUp()
        self.server = xmlrpclib.ServerProxy(
            'http://testserver/xmlrpc/',
            transport=DjangoTestClientTransport(self.client),
        )

    def test_parse_usage(self):
        server = self.server
        today = datetime.datetime.today()
        today = today.strftime("%Y-%m-%d")

        module_file = alogger.tests.examples.__file__
        directory = os.path.abspath(os.path.split(module_file)[0])
        path = os.path.join(directory, "torque.log")
        fd = open(path, "r")
        lines = fd.readlines()
        fd.close()

        with self.assertRaises(xmlrpclib.Fault) as cm:
            server.parse_usage(
                "tango", "aqws12",
                lines, today, 'tango', 'TORQUE')

        self.assertEqual(cm.exception.faultCode, 81)
        self.assertEqual(
            cm.exception.faultString, 'Username and/or password is incorrect')

        proj = Project.objects.get(pid="TestProject1")
        i = Institute.objects.get(name="Example")

        proj2 = Project.objects.create(pid="pMona0041", institute=i)

        p = Person.objects.create(
            username="blair", short_name="B", full_name="Blair",
            institute=i)
        proj.group.members.add(p)
        Account.objects.create(
            username="blair", person=p,
            default_project=proj,
            date_created=datetime.datetime.today())

        p = Person.objects.create(
            username="philipn", short_name="Phil", full_name="Phillip",
            institute=i)
        proj.group.members.add(p)
        proj2.group.members.add(p)
        Account.objects.create(
            username="philipn", person=p,
            default_project=proj,
            date_created=datetime.datetime.today())

        result = server.parse_usage(
            "tango", "aq12ws",
            lines, today, 'tango', 'TORQUE')
        self.assertEqual(
            result[0],
            'Inserted : 16\nUpdated  : 0\nFailed   : 6\nSkiped   : 35')

    def test_parse_usage_alogger(self):
        server = self.server
        today = datetime.datetime.today()
        today = today.strftime("%Y-%m-%d")

        module_file = alogger.tests.examples.__file__
        directory = os.path.abspath(os.path.split(module_file)[0])
        path = os.path.join(directory, "torque.log")
        fd = open(path, "r")
        lines = fd.readlines()
        fd.close()

        proj = Project.objects.get(pid="TestProject1")
        i = Institute.objects.get(name="Example")

        proj2 = Project.objects.create(pid="pMona0041", institute=i)

        p = Person.objects.create(
            username="blair", short_name="B", full_name="Blair",
            institute=i)
        proj.group.members.add(p)
        Account.objects.create(
            username="blair", person=p,
            default_project=proj,
            date_created=datetime.datetime.today())

        p = Person.objects.create(
            username="philipn", short_name="Phil", full_name="Phillip",
            institute=i)
        proj.group.members.add(p)
        proj2.group.members.add(p)
        Account.objects.create(
            username="philipn", person=p,
            default_project=proj,
            date_created=datetime.datetime.today())

        json_array = []

        parser = get_parser("TORQUE")
        for line in lines:
            d = parser.line_to_dict(line)
            if d is None:
                continue

            json_array.append(json.dumps(d))

        result = server.parse_usage(
            "tango", "aq12ws",
            json_array, today, 'tango', 'alogger')
        self.assertEqual(
            result[0],
            'Inserted : 16\nUpdated  : 0\nFailed   : 6\nSkiped   : 0')

    def test_add_modules_used(self):
        server = self.server
        today = datetime.datetime.today()
        today = today.strftime("%Y-%m-%d")

        with self.assertRaises(xmlrpclib.Fault) as cm:
            server.add_modules_used(
                "tango", "aqws12",
                "jobid", [], today)

        self.assertEqual(cm.exception.faultCode, 81)
        self.assertEqual(
            cm.exception.faultString, 'Username and/or password is incorrect')

        result = server.add_modules_used(
            "tango", "aq12ws",
            "942337.tango-m.vpac.org", [
                "modules", "pgi/12.10", "gmp/5.0.5", "mpfr/3.1.1", "mpc/1.0",
                "gcc/4.7.2", "intel/12.1.3", "openmpi-pgi/1.6.3",
                "vpac/config", "lapack/3.4.2", "octave/3.6.3"],
            today
        )
        self.assertEqual(result, True)

        multicall = xmlrpclib.MultiCall(server)

        multicall.add_modules_used(
            "tango", "aq12ws",
            "942337.tango-m.vpac.org", [
                "modules", "pgi/12.10", "gmp/5.0.5", "mpfr/3.1.1", "mpc/1.0",
                "gcc/4.7.2", "intel/12.1.3", "openmpi-pgi/1.6.3",
                "vpac/config", "lapack/3.4.2", "octave/3.6.3"],
            today
        )

        multicall.add_modules_used(
            "tango", "aq12ws",
            "942338.tango-m.vpac.org", [
                "modules", "pgi/12.10", "gmp/5.0.5", "mpfr/3.1.1", "mpc/1.0",
                "gcc/4.7.2", "intel/12.1.3", "openmpi-pgi/1.6.3",
                "vpac/config", "lapack/3.4.2", "octave/3.6.3"],
            today
        )

        result = tuple(multicall())
        self.assertEqual(result, (True, True))



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import pytest
import six
from django.test import TestCase

from karaage.projects.forms import ProjectForm
from karaage.tests.fixtures import ProjectFactory


@pytest.mark.django_db
class ProjectFormTestCase(TestCase):

    def setUp(self):
        super(ProjectFormTestCase, self).setUp()
        self.project = ProjectFactory()

    def _valid_form_data(self):
        data = {
            'pid': self.project.pid,
            'name': self.project.name,
            'description': self.project.description,
            'institute': self.project.institute.id,
            'additional_req': self.project.additional_req,
            'start_date': self.project.start_date,
            'end_date': self.project.end_date
        }
        return data

    def test_valid_data(self):
        form_data = self._valid_form_data()
        form_data['name'] = 'test-project'
        form = ProjectForm(data=form_data,
                           instance=self.project)
        self.assertEqual(form.is_valid(), True, form.errors.items())
        form.save()
        self.assertEqual(self.project.name, 'test-project')

    def test_invalid_pid(self):
        form_data = self._valid_form_data()
        form_data['pid'] = '!test-project'
        form = ProjectForm(data=form_data)
        self.assertEqual(form.is_valid(), False)
        self.assertEqual(
            form.errors.items(),
            dict.items({
                'leaders': [six.u('This field is required.')],
                'pid': [six.u(
                    'Project names can only contain letters,'
                    ' numbers and underscores')]
            })
        )



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

# Hack to allow compatability with versions of django-pipline before 1.4.0
#
# This file can be deleted entirely after backward compatability no longer
# required.

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)


try:
    # django-pipeline 1.4.0
    from pipeline.templatetags.pipeline import *  # NOQA

except ImportError:
    # django-pipeline << 1.4.0
    from django import template
    register = template.Library()

    import pipeline.templatetags.compressed

    register.tag('javascript', pipeline.templatetags.compressed.compressed_js)
    register.tag('stylesheet', pipeline.templatetags.compressed.compressed_css)



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

""" Test LDAP Data"""

GROUP_DN = 'ou=Group, dc=python-ldap,dc=org'
ACCOUNT_DN = 'ou=Account, dc=python-ldap,dc=org'
PEOPLE_DN = 'ou=People, dc=python-ldap,dc=org'
PEOPLE_GROUP_DN = 'ou=PeopleGroup, dc=python-ldap,dc=org'

test_ldif = [
    "dn: " + GROUP_DN,
    "objectClass: organizationalUnit",
    "ou: Groups",
    "",
    "dn: " + ACCOUNT_DN,
    "objectClass: organizationalUnit",
    "ou: Account",
    "",
    "dn: " + PEOPLE_DN,
    "objectClass: organizationalUnit",
    "ou: People",
    "",
    "dn: " + PEOPLE_GROUP_DN,
    "objectClass: organizationalUnit",
    "ou: PeopleGroup",
    "",
    'dn: uid=kgtestuser3, ' + ACCOUNT_DN,
    'cn: Test User3',
    'objectClass: inetOrgPerson',
    'objectClass: person',
    'objectClass: organizationalPerson',
    'objectClass: top',
    'objectClass: shadowAccount',
    'objectClass: posixAccount',
    'objectClass: pwdPolicy',
    'userPassword: kklk',
    'o: Example',
    'sn: User3',
    'mail: t.user3@example.com',
    'givenName: Test',
    'uid: kgtestuser3',
    'shadowWarning: 10',
    'shadowMax: 99999',
    'shadowLastChange: 13600',
    'telephoneNumber: 45645',
    'uidNumber: 100',
    'gidNumber: 500',
    'homeDirectory: /home/kgtestuser3',
    'loginShell: /bin/bash',
    'gecos: Test User3 (Example)',
    'pwdAttribute: userPassword',
    '',
    'dn: uid=kgtestuser3, ' + PEOPLE_DN,
    'cn: Test User3',
    'objectClass: inetOrgPerson',
    'objectClass: person',
    'objectClass: organizationalPerson',
    'objectClass: top',
    'objectClass: pwdPolicy',
    'userPassword: kklk',
    'o: Example',
    'sn: User3',
    'mail: t.user3@example.com',
    'givenName: Test',
    'uid: kgtestuser3',
    'telephoneNumber: 45645',
    'pwdAttribute: userPassword',
    '',
    'dn: uid=kgldaponly, ' + ACCOUNT_DN,
    'cn: LDAP Only',
    'objectClass: inetOrgPerson',
    'objectClass: person',
    'objectClass: organizationalPerson',
    'objectClass: top',
    'objectClass: shadowAccount',
    'objectClass: posixAccount',
    'userPassword: kklk',
    'o: Example',
    'sn: Only',
    'mail: ldaponly@example.com',
    'givenName: LDAP',
    'uid: kgldaponly',
    'shadowWarning: 10',
    'shadowMax: 99999',
    'shadowLastChange: 13600',
    'telephoneNumber: 45645',
    'uidNumber: 100',
    'gidNumber: 500',
    'homeDirectory: /home/kgldaponly',
    'loginShell: /bin/bash',
    'gecos: LDAP Only (Example)',
    '',
    'dn: cn=Example, ' + GROUP_DN,
    'objectClass: posixGroup',
    'objectClass: top',
    'gidNumber: 500',
    'cn: Example',
    'description: Example',
    '',
    'dn: cn=OtherInst, ' + GROUP_DN,
    'objectClass: posixGroup',
    'objectClass: top',
    'gidNumber: 501',
    'cn: OtherInst',
    'description: Example',
    '',
    'dn: cn=SamlInst, ' + GROUP_DN,
    'objectClass: posixGroup',
    'objectClass: top',
    'gidNumber: 502',
    'cn: SamlInst',
    'description: Example',
    '',
    'dn: cn=TestProject1, ' + GROUP_DN,
    'objectClass: posixGroup',
    'objectClass: top',
    'gidNumber: 504',
    'cn: TestProject1',
    'description: TestProject1',
    'memberUid: kgtestuser3',
    '',
    'dn: cn=TestProject1, ' + PEOPLE_GROUP_DN,
    'objectClass: posixGroup',
    'objectClass: top',
    'gidNumber: 504',
    'cn: TestProject1',
    'description: TestProject1',
    'memberUid: kgtestuser3',
    '',
]



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import django_filters
import django_tables2 as tables
import six
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django_tables2.utils import A, AttributeDict

from .models import Group, Person


class ActiveFilter(django_filters.ChoiceFilter):

    def __init__(self, *args, **kwargs):
        choices = [
            ('', 'Unknown'),
            ('deleted', 'Deleted'),
            ('locked', 'Locked'),
            ('yes', 'Yes'),
        ]

        super(ActiveFilter, self).__init__(*args, choices=choices, **kwargs)

    def filter(self, qs, value):
        if value == "deleted":
            qs = qs.filter(date_deleted__isnull=False)
        elif value == "locked":
            qs = qs.filter(date_deleted__isnull=True, login_enabled=False)
        elif value == "yes":
            qs = qs.filter(date_deleted__isnull=True, login_enabled=True)

        return qs


class PeopleColumn(tables.Column):

    def render_link(self, uri, value, attrs=None):
        attrs = AttributeDict(attrs if attrs is not None else
                              self.attrs.get('a', {}))
        attrs['href'] = uri

        return format_html(
            '<a {attrs}>{text}</a>',
            attrs=attrs.as_html(),
            text=value,
        )

    def render(self, value):
        people = []
        for person in value.all():
            url = reverse("kg_person_detail", args=[person.username])
            link = self.render_link(url, value=six.text_type(person))
            people.append(link)
        return mark_safe(", ".join(people))


class PersonFilter(django_filters.FilterSet):
    active = ActiveFilter()
    username = django_filters.CharFilter(lookup_expr="icontains")
    full_name = django_filters.CharFilter(lookup_expr="icontains")
    email = django_filters.CharFilter(lookup_expr="icontains")
    no_last_usage = django_filters.BooleanFilter(
        field_name="last_usage", lookup_expr="isnull")
    begin_last_usage = django_filters.DateFilter(
        field_name="last_usage", lookup_expr="gte")
    end_last_usage = django_filters.DateFilter(
        field_name="last_usage", lookup_expr="lte")
    begin_date_approved = django_filters.DateFilter(
        field_name="date_approved", lookup_expr="gte")
    end_date_approved = django_filters.DateFilter(
        field_name="date_approved", lookup_expr="lte")

    class Meta:
        model = Person
        fields = ("active", "username", "full_name", "email", "institute",
                  "is_admin", )


class PersonTable(tables.Table):
    active = tables.Column(
        empty_values=(), order_by=('date_deleted', '-login_enabled'))
    username = tables.LinkColumn(
        'kg_person_detail', args=[A('username')])
    institute = tables.LinkColumn(
        'kg_institute_detail', args=[A('institute.pk')])

    def render_active(self, record):
        if record.date_deleted is not None:
            html = '<span class="no">Deleted</span>'
        elif not record.login_enabled:
            html = '<span class="locked">Locked</span>'
        else:
            html = '<span class="yes">Yes</span>'
        return mark_safe(html)

    class Meta:
        model = Person
        fields = ("active", "username", "full_name", "institute",
                  "is_admin", "last_usage", "date_approved")
        empty_text = "No items"


class LeaderTable(tables.Table):
    leader = tables.LinkColumn(
        'kg_person_detail', args=[A('leader.username')])
    institute = tables.LinkColumn(
        'kg_institute_detail',
        args=[A('leader.institute.pk')],
        accessor="leader.institute")
    project = tables.LinkColumn(
        'kg_project_detail', args=[A('project.pk')])

    class Meta:
        fields = ("leader", "institute", "project", )
        empty_text = "No items"


class GroupFilter(django_filters.FilterSet):
    name = django_filters.CharFilter(lookup_expr="icontains")
    description = django_filters.CharFilter(lookup_expr="icontains")

    class Meta:
        model = Group
        fields = ("name", "description")


class GroupTable(tables.Table):
    name = tables.LinkColumn(
        'kg_group_detail', args=[A('name')])
    members = PeopleColumn(orderable=False)

    class Meta:
        model = Group
        fields = ("name", "description")
        empty_text = "No items"



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.apps import apps
from django.contrib import messages
from django.contrib.auth import login as auth_login
from django.http import HttpResponseBadRequest, HttpResponseRedirect
from django.shortcuts import render
from django.urls import reverse
from django.views.decorators.debug import sensitive_post_parameters

import karaage.common as common
import karaage.common.saml as saml
from karaage.common.decorators import login_required
from karaage.common.forms import LoginForm
from karaage.people.emails import send_reset_password_email
from karaage.people.forms import PasswordChangeForm, PersonForm
from karaage.people.models import Person


@sensitive_post_parameters('password')
def login(request, username=None):
    error = ''
    redirect_to = reverse('index')
    if 'next' in request.GET:
        redirect_to = request.GET['next']

    if request.POST:

        form = LoginForm(request.POST)
        if form.is_valid():

            username = form.cleaned_data['username']
            password = form.cleaned_data['password']
            person = Person.objects.authenticate(
                username=username, password=password)
            if person is not None:
                if person.is_active and not person.is_locked():
                    auth_login(request, person)
                    return HttpResponseRedirect(redirect_to)
                else:
                    error = 'User account is inactive or locked'
            else:
                error = 'Username or password was incorrect'
    else:
        form = LoginForm(initial={'username': username})

    return render(
        template_name='karaage/people/profile_login.html',
        context={
            'form': form,
            'next': redirect_to,
            'error': error,
        }, request=request)


def saml_login(request):
    redirect_to = reverse('index')
    if 'next' in request.GET:
        redirect_to = request.GET['next']
    error = None
    saml_session = saml.is_saml_session(request)

    form = saml.SAMLInstituteForm(request.POST or None)
    if request.method == 'POST':
        if 'login' in request.POST and form.is_valid():
            institute = form.cleaned_data['institute']
            url = saml.build_shib_url(
                request, redirect_to,
                institute.saml_entityid)
            return HttpResponseRedirect(url)
        elif 'logout' in request.POST:
            if saml_session:
                url = saml.logout_url(request)
                return HttpResponseRedirect(url)
            else:
                return HttpResponseBadRequest("<h1>Bad Request</h1>")
        else:
            return HttpResponseBadRequest("<h1>Bad Request</h1>")
    elif request.user.is_authenticated:
        error = "You are already logged in."
    elif saml_session:
        attrs, error = saml.parse_attributes(request)
        saml_id = attrs['persistent_id']
        try:
            Person.objects.get(saml_id=saml_id)
            # This should not happen, suggests a fault in the saml middleware
            error = "Shibboleth session established " \
                    "but you did not get logged in. "
        except Person.DoesNotExist:
            email = attrs['email']
            try:
                Person.objects.get(email=email)
                error = "Cannot log in with this shibboleth account. " \
                        "Please try using the Karaage login instead."
            except Person.DoesNotExist:
                if apps.is_installed("karaage.plugins.kgapplications"):
                    app_url = reverse('kg_application_new')
                    return HttpResponseRedirect(app_url)
                else:
                    error = "Cannot log in with shibboleth as " \
                            "we do not recognise your shibboleth id."

    return render(
        template_name='karaage/people/profile_login_saml.html',
        context={'form': form, 'error': error, 'saml_session': saml_session, },
        request=request)


def saml_details(request):
    redirect_to = reverse('kg_profile_saml')
    saml_session = saml.is_saml_session(request)

    if request.method == 'POST':
        if 'login' in request.POST:
            if request.user.is_authenticated:
                person = request.user
                institute = person.institute
                if institute.saml_entityid:
                    url = saml.build_shib_url(
                        request, redirect_to,
                        institute.saml_entityid)
                    return HttpResponseRedirect(url)
                else:
                    return HttpResponseBadRequest("<h1>Bad Request</h1>")
            else:
                return HttpResponseBadRequest("<h1>Bad Request</h1>")

        elif 'register' in request.POST:
            if request.user.is_authenticated and saml_session:
                person = request.user
                person = saml.add_saml_data(
                    person, request)
                person.save()
                return HttpResponseRedirect(redirect_to)
            else:
                return HttpResponseBadRequest("<h1>Bad Request</h1>")

        elif 'logout' in request.POST:
            if saml_session:
                url = saml.logout_url(request)
                return HttpResponseRedirect(url)
            else:
                return HttpResponseBadRequest("<h1>Bad Request</h1>")

        else:
            return HttpResponseBadRequest("<h1>Bad Request</h1>")

    attrs = {}
    if saml_session:
        attrs, _ = saml.parse_attributes(request)
        saml_session = True

    person = None
    if request.user.is_authenticated:
        person = request.user

    return render(
        template_name='karaage/people/profile_saml.html',
        context={
            'attrs': attrs, 'saml_session': saml_session, 'person': person, },
        request=request)


@login_required
def profile_personal(request):

    person = request.user
    project_list = person.projects.all()
    project_requests = []
    user_applications = []
    start, end = common.get_date_range(request)

    return render(
        template_name='karaage/people/profile_personal.html',
        context=locals(),
        request=request)


@login_required
def edit_profile(request):
    person = request.user
    form = PersonForm(request.POST or None, instance=person)
    if request.method == 'POST':
        if form.is_valid():
            person = form.save()
            assert person is not None
            messages.success(
                request, "User '%s' was edited succesfully" % person)
            return HttpResponseRedirect(person.get_absolute_url())

    return render(
        template_name='karaage/people/profile_edit.html',
        context={'person': person, 'form': form},
        request=request)


@sensitive_post_parameters('new1', 'new2')
@login_required
def password_change(request):

    person = request.user

    if request.POST:
        form = PasswordChangeForm(data=request.POST, person=person)

        if form.is_valid():
            form.save()
            messages.success(request, "Password changed successfully")
            return HttpResponseRedirect(reverse('kg_profile'))
    else:
        form = PasswordChangeForm(person=person)

    return render(
        template_name='karaage/common/profile_password.html',
        context={'person': person, 'form': form},
        request=request)


@login_required
def password_request(request):
    person = request.user

    post_reset_redirect = reverse('kg_profile_reset_done')

    if request.method == "POST":
        send_reset_password_email(person)
        return HttpResponseRedirect(post_reset_redirect)

    var = {
        'person': person,
    }
    return render(
        template_name='karaage/common/profile_password_request.html',
        context=var,
        request=request)


@login_required
def password_request_done(request):
    person = request.user
    var = {
        'person': person,
    }
    return render(
        template_name='karaage/common/profile_password_request_done.html',
        context=var,
        request=request)



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.contrib.auth.decorators import user_passes_test
from django.core.exceptions import PermissionDenied


if settings.SHIB_SUPPORTED:
    _login_url = "kg_profile_login_saml"
else:
    _login_url = "kg_profile_login"


def admin_required(function=None):
    """
    Decorator for views that checks that the user is an administrator,
    redirecting to the log-in page if necessary.
    """
    def check_perms(user):
        # if user not logged in, show login form
        if not user.is_authenticated:
            return False
        # if this site doesn't allow admin access, fail
        if settings.ADMIN_IGNORED:
            raise PermissionDenied
        # check if the user has admin rights
        if not user.is_admin:
            raise PermissionDenied
        return True

    actual_decorator = user_passes_test(check_perms, login_url=_login_url)
    if function:
        return actual_decorator(function)
    return actual_decorator


def login_required(function=None):
    """
    Decorator for views that checks that the user is logged in, redirecting
    to the log-in page if necessary.
    """
    def check_perms(user):
        # if user not logged in, show login form
        if not user.is_authenticated:
            return False
        # if this is the admin site only admin access
        if settings.ADMIN_REQUIRED and not user.is_admin:
            raise PermissionDenied
        return True

    actual_decorator = user_passes_test(check_perms, login_url=_login_url)
    if function:
        return actual_decorator(function)
    return actual_decorator


def usage_required(function=None):
    """
    Decorator for views that checks that the user is logged in, redirecting
    to the log-in page if necessary.
    """
    def check_perms(user):
        # if user not logged in, show login form
        if not user.is_authenticated:
            return False
        # if this is the admin site only admin access
        if settings.ADMIN_REQUIRED and not user.is_admin:
            raise PermissionDenied
        if settings.USAGE_IS_PUBLIC:
            return True
        if user.is_admin:
            return True
        return False

    actual_decorator = user_passes_test(check_perms, login_url=_login_url)
    if function:
        return actual_decorator(function)
    return actual_decorator


def xmlrpc_machine_required(function=None):
    """
    Decorator for views that checks that the user is logged in, redirecting
    to the log-in page if necessary.
    """
    def actual_decorator(func):
        def wrapper(machine_name, password, *args):
            from django_xmlrpc.decorators import AuthenticationFailedException
            from karaage.machines.models import Machine
            machine = Machine.objects.authenticate(machine_name, password)
            if machine is None:
                raise AuthenticationFailedException
            return func(machine, *args)

        if hasattr(func, '_xmlrpc_signature'):
            sig = func._xmlrpc_signature
            sig['args'] = (['string'] * 2) + sig['args']
            wrapper._xmlrpc_signature = sig

        if func.__doc__:
            wrapper.__doc__ = func.__doc__ + \
                "\nNote: Machine authentication is required."
        return wrapper

    if function:
        return actual_decorator(function)
    return actual_decorator



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import pytest
import six
from django.conf import settings
from django.test import TestCase

from karaage.machines.forms import AdminAccountForm
from karaage.tests.fixtures import simple_account


@pytest.mark.django_db
class AdminAccountFormTestCase(TestCase):

    def setUp(self):
        super(AdminAccountFormTestCase, self).setUp()
        self.account = simple_account()

    def _valid_form_data(self):
        text = six.text_type
        data = {
            'username': self.account.username,
            'default_project': text(self.account.default_project.id),
            'shell': settings.DEFAULT_SHELL,
        }
        return data

    def test_valid_data(self):
        form_data = self._valid_form_data()
        form_data['username'] = 'test-account'
        form = AdminAccountForm(person=self.account.person,
                                data=form_data,
                                instance=self.account)
        self.assertEqual(form.is_valid(), True, form.errors.items())
        form.save()
        self.assertEqual(self.account.username, 'test-account')

    def test_invalid_usernamen(self):
        form_data = self._valid_form_data()
        form_data['username'] = '!test-account'
        form = AdminAccountForm(person=self.account.person,
                                data=form_data,
                                instance=self.account)
        self.assertEqual(form.is_valid(), False)
        self.assertEqual(
            form.errors.items(),
            dict.items({
                'username': [(six.u(
                    'Usernames can only contain '
                    'letters, numbers and underscores'))]
            })
        )

    def test_upper_username(self):
        form_data = self._valid_form_data()
        form_data['username'] = 'INVALID'
        form = AdminAccountForm(person=self.account.person,
                                data=form_data,
                                instance=self.account)
        self.assertEqual(form.is_valid(), False)
        self.assertEqual(
            form.errors.items(),
            dict.items({
                'username': [six.u('Username must be all lowercase')]
            })
        )

    def test_long_username(self):
        form_data = self._valid_form_data()
        form_data['username'] = 'long' * 100
        form = AdminAccountForm(person=self.account.person,
                                data=form_data,
                                instance=self.account)
        self.assertEqual(form.is_valid(), False)
        self.assertEqual(
            form.errors.items(),
            dict.items({
                'username': [six.u(
                    'Ensure this value has at '
                    'most 255 characters (it has 400).')]
            })
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
import jsonfield.fields
import datetime


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Person',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(default=django.utils.timezone.now, verbose_name='last login')),
                ('username', models.CharField(unique=True, max_length=255)),
                ('email', models.EmailField(max_length=75, null=True, db_index=True)),
                ('short_name', models.CharField(max_length=30)),
                ('full_name', models.CharField(max_length=60)),
                ('is_active', models.BooleanField(default=True)),
                ('is_admin', models.BooleanField(default=False)),
                ('saml_id', models.CharField(max_length=200, unique=True, null=True, editable=False, blank=True)),
                ('position', models.CharField(max_length=200, null=True, blank=True)),
                ('telephone', models.CharField(max_length=200, null=True, blank=True)),
                ('mobile', models.CharField(max_length=200, null=True, blank=True)),
                ('department', models.CharField(max_length=200, null=True, blank=True)),
                ('supervisor', models.CharField(max_length=100, null=True, blank=True)),
                ('title', models.CharField(blank=True, max_length=10, null=True, choices=[('', ''), ('Mr', 'Mr'), ('Mrs', 'Mrs'), ('Miss', 'Miss'), ('Ms', 'Ms'), ('Dr', 'Dr'), ('Prof', 'Prof'), ('A/Prof', 'A/Prof')])),
                ('address', models.CharField(max_length=200, null=True, blank=True)),
                ('city', models.CharField(max_length=100, null=True, blank=True)),
                ('postcode', models.CharField(max_length=8, null=True, blank=True)),
                ('state', models.CharField(blank=True, max_length=4, null=True, choices=[('', '--------'), ('ACT', 'ACT'), ('NSW', 'New South Wales'), ('NT', 'Northern Territory'), ('QLD', 'Queensland'), ('SA', 'South Australia'), ('TAS', 'Tasmania'), ('VIC', 'Victoria'), ('WA', 'Western Australia')])),
                ('country', models.CharField(blank=True, max_length=2, null=True, choices=[('AU', 'Australia'), ('NZ', 'New Zealand'), ('GB', 'United Kingdom'), ('DE', 'Germany'), ('US', 'United States'), ('', '--------------------------------------'), ('AD', 'Andorra'), ('AE', 'United Arab Emirates'), ('AF', 'Afghanistan'), ('AG', 'Antigua and Barbuda'), ('AI', 'Anguilla'), ('AL', 'Albania'), ('AM', 'Armenia'), ('AN', 'Netherlands Antilles'), ('AO', 'Angola'), ('AQ', 'Antarctica'), ('AR', 'Argentina'), ('AS', 'American Samoa'), ('AT', 'Austria'), ('AW', 'Aruba'), ('AX', 'Aland Islands'), ('AZ', 'Azerbaijan'), ('BA', 'Bosnia and Herzegovina'), ('BB', 'Barbados'), ('BD', 'Bangladesh'), ('BE', 'Belgium'), ('BF', 'Burkina Faso'), ('BG', 'Bulgaria'), ('BH', 'Bahrain'), ('BI', 'Burundi'), ('BJ', 'Benin'), ('BM', 'Bermuda'), ('BN', 'Brunei Darussalam'), ('BO', 'Bolivia'), ('BR', 'Brazil'), ('BS', 'Bahamas'), ('BT', 'Bhutan'), ('BV', 'Bouvet Island'), ('BW', 'Botswana'), ('BY', 'Belarus'), ('BZ', 'Belize'), ('CA', 'Canada'), ('CC', 'Cocos (Keeling) Islands'), ('CD', 'Congo'), ('CF', 'Central African Republic'), ('CG', 'Congo'), ('CH', 'Switzerland'), ('CI', "Cote d'Ivoire"), ('CK', 'Cook Islands'), ('CL', 'Chile'), ('CM', 'Cameroon'), ('CN', 'China'), ('CO', 'Colombia'), ('CR', 'Costa Rica'), ('CU', 'Cuba'), ('CV', 'Cape Verde'), ('CX', 'Christmas Island'), ('CY', 'Cyprus'), ('CZ', 'Czech Republic'), ('DJ', 'Djibouti'), ('DK', 'Denmark'), ('DM', 'Dominica'), ('DO', 'Dominican Republic'), ('DZ', 'Algeria'), ('EC', 'Ecuador'), ('EE', 'Estonia'), ('EG', 'Egypt'), ('EH', 'Western Sahara'), ('ER', 'Eritrea'), ('ES', 'Spain'), ('ET', 'Ethiopia'), ('FI', 'Finland'), ('FJ', 'Fiji'), ('FK', 'Falkland Islands'), ('FM', 'Micronesia'), ('FO', 'Faroe Islands'), ('FR', 'France'), ('GA', 'Gabon'), ('GD', 'Grenada'), ('GE', 'Georgia'), ('GF', 'French Guiana'), ('GG', 'Guernsey'), ('GH', 'Ghana'), ('GI', 'Gibraltar'), ('GL', 'Greenland'), ('GM', 'Gambia'), ('GN', 'Guinea'), ('GP', 'Guadeloupe'), ('GQ', 'Equatorial Guinea'), ('GR', 'Greece'), ('GS', 'South Georgia and the South Sandwich Islands'), ('GT', 'Guatemala'), ('GU', 'Guam'), ('GW', 'Guinea-Bissau'), ('GY', 'Guyana'), ('HK', 'Hong Kong'), ('HM', 'Heard Island and McDonald Islands'), ('HN', 'Honduras'), ('HR', 'Croatia'), ('HT', 'Haiti'), ('HU', 'Hungary'), ('ID', 'Indonesia'), ('IE', 'Ireland'), ('IL', 'Israel'), ('IM', 'Isle of Man'), ('IN', 'India'), ('IO', 'British Indian Ocean Territory'), ('IQ', 'Iraq'), ('IR', 'Iran'), ('IS', 'Iceland'), ('IT', 'Italy'), ('JE', 'Jersey'), ('JM', 'Jamaica'), ('JO', 'Jordan'), ('JP', 'Japan'), ('KE', 'Kenya'), ('KG', 'Kyrgyzstan'), ('KH', 'Cambodia'), ('KI', 'Kiribati'), ('KM', 'Comoros'), ('KN', 'Saint Kitts and Nevis'), ('KP', 'Korea'), ('KR', 'Korea'), ('KW', 'Kuwait'), ('KY', 'Cayman Islands'), ('KZ', 'Kazakhstan'), ('LA', "Lao People's Democratic Republic"), ('LB', 'Lebanon'), ('LC', 'Saint Lucia'), ('LI', 'Liechtenstein'), ('LK', 'Sri Lanka'), ('LR', 'Liberia'), ('LS', 'Lesotho'), ('LT', 'Lithuania'), ('LU', 'Luxembourg'), ('LV', 'Latvia'), ('LY', 'Libyan Arab Jamahiriya'), ('MA', 'Morocco'), ('MC', 'Monaco'), ('MD', 'Moldova'), ('ME', 'Montenegro'), ('MG', 'Madagascar'), ('MH', 'Marshall Islands'), ('MK', 'Macedonia'), ('ML', 'Mali'), ('MM', 'Myanmar'), ('MN', 'Mongolia'), ('MO', 'Macao'), ('MP', 'Northern Mariana Islands'), ('MQ', 'Martinique'), ('MR', 'Mauritania'), ('MS', 'Montserrat'), ('MT', 'Malta'), ('MU', 'Mauritius'), ('MV', 'Maldives'), ('MW', 'Malawi'), ('MX', 'Mexico'), ('MY', 'Malaysia'), ('MZ', 'Mozambique'), ('NA', 'Namibia'), ('NC', 'New Caledonia'), ('NE', 'Niger'), ('NF', 'Norfolk Island'), ('NG', 'Nigeria'), ('NI', 'Nicaragua'), ('NL', 'Netherlands'), ('NO', 'Norway'), ('NP', 'Nepal'), ('NR', 'Nauru'), ('NU', 'Niue'), ('OM', 'Oman'), ('PA', 'Panama'), ('PE', 'Peru'), ('PF', 'French Polynesia'), ('PG', 'Papua New Guinea'), ('PH', 'Philippines'), ('PK', 'Pakistan'), ('PL', 'Poland'), ('PM', 'Saint Pierre and Miquelon'), ('PN', 'Pitcairn'), ('PR', 'Puerto Rico'), ('PS', 'Palestinian Territory'), ('PT', 'Portugal'), ('PW', 'Palau'), ('PY', 'Paraguay'), ('QA', 'Qatar'), ('RE', 'Reunion'), ('RO', 'Romania'), ('RS', 'Serbia'), ('RU', 'Russian Federation'), ('RW', 'Rwanda'), ('SA', 'Saudi Arabia'), ('SB', 'Solomon Islands'), ('SC', 'Seychelles'), ('SD', 'Sudan'), ('SE', 'Sweden'), ('SG', 'Singapore'), ('SH', 'Saint Helena'), ('SI', 'Slovenia'), ('SJ', 'Svalbard and Jan Mayen'), ('SK', 'Slovakia'), ('SL', 'Sierra Leone'), ('SM', 'San Marino'), ('SN', 'Senegal'), ('SO', 'Somalia'), ('SR', 'Suriname'), ('ST', 'Sao Tome and Principe'), ('SV', 'El Salvador'), ('SY', 'Syrian Arab Republic'), ('SZ', 'Swaziland'), ('TC', 'Turks and Caicos Islands'), ('TD', 'Chad'), ('TF', 'French Southern Territories'), ('TG', 'Togo'), ('TH', 'Thailand'), ('TJ', 'Tajikistan'), ('TK', 'Tokelau'), ('TL', 'Timor-Leste'), ('TM', 'Turkmenistan'), ('TN', 'Tunisia'), ('TO', 'Tonga'), ('TR', 'Turkey'), ('TT', 'Trinidad and Tobago'), ('TV', 'Tuvalu'), ('TW', 'Taiwan'), ('TZ', 'Tanzania'), ('UA', 'Ukraine'), ('UG', 'Uganda'), ('UM', 'United States Minor Outlying Islands'), ('UY', 'Uruguay'), ('UZ', 'Uzbekistan'), ('VA', 'Vatican City'), ('VC', 'Saint Vincent and the Grenadines'), ('VE', 'Venezuela'), ('VG', 'Virgin Islands (British)'), ('VI', 'Virgin Islands (US)'), ('VN', 'Viet Nam'), ('VU', 'Vanuatu'), ('WF', 'Wallis and Futuna'), ('WS', 'Samoa'), ('YE', 'Yemen'), ('YT', 'Mayotte'), ('ZA', 'South Africa'), ('ZM', 'Zambia'), ('ZW', 'Zimbabwe')])),
                ('website', models.URLField(null=True, blank=True)),
                ('fax', models.CharField(max_length=50, null=True, blank=True)),
                ('comment', models.TextField(null=True, blank=True)),
                ('date_approved', models.DateField(null=True, blank=True)),
                ('date_deleted', models.DateField(null=True, blank=True)),
                ('last_usage', models.DateField(null=True, blank=True)),
                ('expires', models.DateField(null=True, blank=True)),
                ('is_systemuser', models.BooleanField(default=False)),
                ('login_enabled', models.BooleanField(default=True)),
                ('legacy_ldap_password', models.CharField(max_length=128, null=True, blank=True)),
                ('approved_by', models.ForeignKey(related_name='user_approver', blank=True, to='karaage.Person', null=True)),
                ('deleted_by', models.ForeignKey(related_name='user_deletor', blank=True, to='karaage.Person', null=True)),
            ],
            options={
                'ordering': ['full_name', 'short_name'],
                'db_table': 'person',
                'verbose_name_plural': 'people',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('username', models.CharField(max_length=255)),
                ('foreign_id', models.CharField(help_text='The foreign identifier from the datastore.', max_length=255, unique=True, null=True)),
                ('date_created', models.DateField()),
                ('date_deleted', models.DateField(null=True, blank=True)),
                ('disk_quota', models.IntegerField(help_text='In GB', null=True, blank=True)),
                ('shell', models.CharField(max_length=50)),
                ('login_enabled', models.BooleanField(default=True)),
                ('extra_data', jsonfield.fields.JSONField(default={}, help_text='Datastore specific values should be stored in this field.')),
            ],
            options={
                'ordering': ['person'],
                'db_table': 'account',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(unique=True, max_length=255)),
                ('foreign_id', models.CharField(help_text='The foreign identifier from the datastore.', max_length=255, unique=True, null=True)),
                ('description', models.TextField(null=True, blank=True)),
                ('extra_data', jsonfield.fields.JSONField(default={}, help_text='Datastore specific values should be stored in this field.')),
                ('members', models.ManyToManyField(related_name='groups', to='karaage.Person')),
            ],
            options={
                'ordering': ['name'],
                'db_table': 'people_group',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Institute',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(unique=True, max_length=255)),
                ('saml_entityid', models.CharField(max_length=200, unique=True, null=True, blank=True)),
                ('is_active', models.BooleanField(default=True)),
            ],
            options={
                'ordering': ['name'],
                'db_table': 'institute',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='InstituteDelegate',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('send_email', models.BooleanField()),
                ('institute', models.ForeignKey(to='karaage.Institute')),
                ('person', models.ForeignKey(to='karaage.Person')),
            ],
            options={
                'db_table': 'institutedelegate',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='InstituteQuota',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('quota', models.DecimalField(max_digits=5, decimal_places=2)),
                ('cap', models.IntegerField(null=True, blank=True)),
                ('disk_quota', models.IntegerField(null=True, blank=True)),
                ('institute', models.ForeignKey(to='karaage.Institute')),
            ],
            options={
                'db_table': 'institute_quota',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='LogEntry',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('action_time', models.DateTimeField(auto_now_add=True, verbose_name='action time')),
                ('object_id', models.TextField(null=True, verbose_name='object id', blank=True)),
                ('object_repr', models.CharField(max_length=200, verbose_name='object repr')),
                ('action_flag', models.PositiveSmallIntegerField(verbose_name='action flag')),
                ('change_message', models.TextField(verbose_name='change message', blank=True)),
                ('content_type', models.ForeignKey(blank=True, to='contenttypes.ContentType', null=True)),
                ('user', models.ForeignKey(to='karaage.Person', null=True)),
            ],
            options={
                'ordering': ('-action_time', '-pk'),
                'db_table': 'admin_log',
                'verbose_name': 'log entry',
                'verbose_name_plural': 'log entries',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Machine',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(default=django.utils.timezone.now, verbose_name='last login')),
                ('name', models.CharField(unique=True, max_length=50)),
                ('no_cpus', models.IntegerField()),
                ('no_nodes', models.IntegerField()),
                ('type', models.CharField(max_length=100)),
                ('start_date', models.DateField()),
                ('end_date', models.DateField(null=True, blank=True)),
                ('pbs_server_host', models.CharField(max_length=50, null=True, blank=True)),
                ('mem_per_core', models.IntegerField(help_text='In GB', null=True, blank=True)),
                ('scaling_factor', models.IntegerField(default=1)),
            ],
            options={
                'db_table': 'machine',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MachineCategory',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(unique=True, max_length=100)),
                ('datastore', models.CharField(help_text='Modifying this value on existing categories will affect accounts created under the old datastore', max_length=255, choices=[('dummy', 'dummy'), ('ldap', 'ldap')])),
            ],
            options={
                'db_table': 'machine_category',
                'verbose_name_plural': 'machine categories',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Project',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('pid', models.CharField(unique=True, max_length=255)),
                ('name', models.CharField(max_length=200)),
                ('description', models.TextField(null=True, blank=True)),
                ('is_approved', models.BooleanField(default=False)),
                ('start_date', models.DateField(default=datetime.datetime.today)),
                ('end_date', models.DateField(null=True, blank=True)),
                ('additional_req', models.TextField(null=True, blank=True)),
                ('is_active', models.BooleanField(default=False)),
                ('date_approved', models.DateField(null=True, editable=False, blank=True)),
                ('date_deleted', models.DateField(null=True, editable=False, blank=True)),
                ('last_usage', models.DateField(null=True, editable=False, blank=True)),
                ('approved_by', models.ForeignKey(related_name='project_approver', blank=True, editable=False, to='karaage.Person', null=True)),
                ('deleted_by', models.ForeignKey(related_name='project_deletor', blank=True, editable=False, to='karaage.Person', null=True)),
                ('group', models.ForeignKey(to='karaage.Group')),
                ('institute', models.ForeignKey(to='karaage.Institute')),
                ('leaders', models.ManyToManyField(related_name='leads', to='karaage.Person')),
            ],
            options={
                'ordering': ['pid'],
                'db_table': 'project',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ProjectQuota',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('cap', models.IntegerField(null=True, blank=True)),
                ('machine_category', models.ForeignKey(to='karaage.MachineCategory')),
                ('project', models.ForeignKey(to='karaage.Project')),
            ],
            options={
                'db_table': 'project_quota',
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='projectquota',
            unique_together=set([('project', 'machine_category')]),
        ),
        migrations.AddField(
            model_name='machine',
            name='category',
            field=models.ForeignKey(to='karaage.MachineCategory'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='institutequota',
            name='machine_category',
            field=models.ForeignKey(to='karaage.MachineCategory'),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='institutequota',
            unique_together=set([('institute', 'machine_category')]),
        ),
        migrations.AddField(
            model_name='institute',
            name='delegates',
            field=models.ManyToManyField(related_name='delegate_for', null=True, through='karaage.InstituteDelegate', to='karaage.Person', blank=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='institute',
            name='group',
            field=models.ForeignKey(to='karaage.Group'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='account',
            name='default_project',
            field=models.ForeignKey(blank=True, to='karaage.Project', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='account',
            name='machine_category',
            field=models.ForeignKey(to='karaage.MachineCategory'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='account',
            name='person',
            field=models.ForeignKey(to='karaage.Person'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='person',
            name='institute',
            field=models.ForeignKey(to='karaage.Institute'),
            preserve_default=True,
        ),
    ]



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.conf.urls import url

from karaage.people.views import groups


urlpatterns = [
    url(r'^$', groups.group_detail, name='kg_group_detail'),
    url(r'^verbose/$', groups.group_verbose, name='kg_group_verbose'),
    url(r'^delete/$', groups.delete_group, name='kg_group_delete'),
    url(r'^add/$', groups.add_group_member, name='kg_group_add_person'),
    url(r'^remove/(?P<username>%s)/$' % settings.USERNAME_VALIDATION_RE,
        groups.remove_group_member, name='kg_group_remove_person'),
    url(r'^logs/$', groups.group_logs, name='kg_group_logs'),
    url(r'^add_comment/$', groups.add_comment, name='kg_group_add_comment'),
    url(r'^edit/$', groups.edit_group, name='kg_group_edit'),
]



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

"""Test all pages render (without exceptions)."""

from __future__ import print_function, unicode_literals


# from karaage.tests.client import TestAllPagesCase


# import pytest
# @pytest.mark.django_db
# class TestKgApplicationPages(TestAllPagesCase):
#
#     """Discover all URLs, do a HTTP GET and
#     confirm 200 OK and no DB changes."""
#
#     fixtures = [
#         'test_karaage.json',
#         'test_kgsoftware.json',
#         'test_kgusage.json'
#     ]
#     variables = {
#         # 'jobid': 'dalek001',
#         'software_id': '1',
#         'version_id': '1',
#         'institute_id': '1',
#         'project_id': '1',
#     }
#     module = "karaage.plugins.kgusage"



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import django.db.transaction
import tldap.transaction
from django.conf import settings
from django.core.management.base import BaseCommand

from karaage.people.models import Person


class Command(BaseCommand):
    help = "Lock all training accounts"

    @django.db.transaction.atomic
    @tldap.transaction.commit_on_success
    def handle(self, **options):

        verbose = int(options.get('verbosity'))
        training_prefix = getattr(settings, 'TRAINING_ACCOUNT_PREFIX', 'train')

        # If training accounts are system users, they will be found by
        # Person.objects.all() but not Person.active.all()
        query = Person.objects.all()
        query = query.filter(username__iregex=training_prefix)
        query = query.order_by('username')

        for person in query.all():
            try:
                person.lock()
                if verbose > 1:
                    print("%s: Locked" % person.username)
            except Exception as e:
                print("%s: Failed to lock: %s" % (person, e))



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible
from model_utils import FieldTracker

from karaage.common import is_admin, log
from karaage.institutes.managers import ActiveInstituteManager
from karaage.machines.models import Account
from karaage.people.models import Group, Person


@python_2_unicode_compatible
class Institute(models.Model):
    name = models.CharField(max_length=255, unique=True)
    delegates = models.ManyToManyField(
        Person, related_name='delegate_for',
        blank=True, through='InstituteDelegate')
    group = models.ForeignKey(Group, on_delete=models.PROTECT)
    saml_entityid = models.CharField(
        max_length=200,
        null=True, blank=True, unique=True)
    is_active = models.BooleanField(default=True)
    objects = models.Manager()
    active = ActiveInstituteManager()

    _tracker = FieldTracker()

    class Meta:
        ordering = ['name']
        db_table = 'institute'
        app_label = 'karaage'

    def save(self, *args, **kwargs):
        created = self.pk is None

        # save the object
        super(Institute, self).save(*args, **kwargs)

        if created:
            log.add(self, 'Created')
        for field in self._tracker.changed():
            log.change(self, 'Changed %s to %s'
                       % (field, getattr(self, field)))

        # update the datastore
        from karaage.datastores import save_institute
        save_institute(self)

        # has group changed?
        if self._tracker.has_changed("group_id"):
            old_group_pk = self._tracker.previous("group_id")
            new_group = self.group
            if old_group_pk is not None:
                old_group = Group.objects.get(pk=old_group_pk)
                from karaage.datastores import remove_accounts_from_institute
                query = Account.objects.filter(person__groups=old_group)
                remove_accounts_from_institute(query, self)
            if new_group is not None:
                from karaage.datastores import add_accounts_to_institute
                query = Account.objects.filter(person__groups=new_group)
                add_accounts_to_institute(query, self)
    save.alters_data = True

    def delete(self, *args, **kwargs):
        # Get list of accounts.
        # This must happen before we call the super method,
        # as this will delete accounts that use this institute.
        old_group_pk = self._tracker.previous("group_id")
        if old_group_pk is not None:
            old_group = Group.objects.get(pk=old_group_pk)
            query = Account.objects.filter(person__groups=old_group)
            query = query.filter(date_deleted__isnull=True)
            accounts = list(query)
        else:
            accounts = []

        # delete the object
        log.delete(self, 'Deleted')
        super(Institute, self).delete(*args, **kwargs)

        # update datastore associations
        for account in accounts:
            from karaage.datastores import remove_account_from_institute
            remove_account_from_institute(account, self)

        # update the datastore
        from karaage.datastores import delete_institute
        delete_institute(self)
    delete.alters_data = True

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('kg_institute_detail', args=[self.id])

    def can_view(self, request):
        person = request.user

        if not person.is_authenticated:
            return False

        # staff members can view everything
        if is_admin(request):
            return True

        if not self.is_active:
            return False

        if not person.is_active:
            return False

        if person.is_locked():
            return False

        # Institute delegates==person can view institute
        if person in self.delegates.all():
            return True

        return False


class InstituteDelegate(models.Model):
    person = models.ForeignKey(Person, on_delete=models.CASCADE)
    institute = models.ForeignKey(Institute, on_delete=models.CASCADE)
    send_email = models.BooleanField()

    _tracker = FieldTracker()

    class Meta:
        db_table = 'institutedelegate'
        app_label = 'karaage'

    def save(self, *args, **kwargs):
        super(InstituteDelegate, self).save(*args, **kwargs)

        for field in self._tracker.changed():
            log.change(
                self.institute,
                'Delegate %s: Changed %s to %s' %
                (self.person, field, getattr(self, field)))

    def delete(self, *args, **kwargs):
        super(InstituteDelegate, self).delete(*args, **kwargs)

        log.delete(
            self.institute,
            'Delegate %s: Deleted' % self.person)



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

"""
All email sending is done from this module
"""
from django.conf import settings
from django.core.mail import EmailMessage


CONTEXT = {
    'org_email': settings.ACCOUNTS_EMAIL,
    'org_name': settings.ACCOUNTS_ORG_NAME,
}


def send_mail(subject, message, from_email, recipient_list):
    headers = {
        'Precedence': 'bulk',
        'Auto-Submitted': 'auto-replied',
    }

    email = EmailMessage(subject, message, from_email, recipient_list,
                         headers=headers)
    return email.send()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('karaage', '0001_initial'),
        ('kgsoftware', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CPUJob',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('username', models.CharField(max_length=50, null=True, blank=True)),
                ('date', models.DateField(db_index=True, null=True, blank=True)),
                ('cpu_usage', models.BigIntegerField(null=True, blank=True)),
                ('mem', models.BigIntegerField(null=True, blank=True)),
                ('vmem', models.BigIntegerField(null=True, blank=True)),
                ('ctime', models.DateTimeField(null=True, blank=True)),
                ('qtime', models.DateTimeField(null=True, blank=True)),
                ('etime', models.DateTimeField(null=True, blank=True)),
                ('start', models.DateTimeField(null=True, blank=True)),
                ('act_wall_time', models.BigIntegerField(null=True, blank=True)),
                ('est_wall_time', models.BigIntegerField(null=True, blank=True)),
                ('jobid', models.CharField(max_length=50, unique=True, null=True, blank=True)),
                ('cores', models.BigIntegerField(null=True, blank=True)),
                ('list_mem', models.BigIntegerField(null=True, blank=True)),
                ('list_pmem', models.BigIntegerField(null=True, blank=True)),
                ('list_vmem', models.BigIntegerField(null=True, blank=True)),
                ('list_pvmem', models.BigIntegerField(null=True, blank=True)),
                ('exit_status', models.BigIntegerField(null=True, blank=True)),
                ('jobname', models.CharField(max_length=256, null=True, blank=True)),
                ('account', models.ForeignKey(blank=True, to='karaage.Account', null=True)),
                ('machine', models.ForeignKey(blank=True, to='karaage.Machine', null=True)),
                ('project', models.ForeignKey(blank=True, to='karaage.Project', null=True)),
            ],
            options={
                'ordering': ['-date'],
                'db_table': 'cpu_job',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='InstituteCache',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date', models.DateField(auto_now_add=True)),
                ('start', models.DateField()),
                ('end', models.DateField()),
                ('cpu_time', models.DecimalField(max_digits=30, decimal_places=2)),
                ('no_jobs', models.IntegerField()),
                ('institute', models.ForeignKey(to='karaage.Institute')),
                ('machine_category', models.ForeignKey(to='karaage.MachineCategory')),
            ],
            options={
                'db_table': 'cache_institutecache',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MachineCache',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date', models.DateField(auto_now_add=True)),
                ('start', models.DateField()),
                ('end', models.DateField()),
                ('cpu_time', models.DecimalField(max_digits=30, decimal_places=2)),
                ('no_jobs', models.IntegerField()),
                ('machine', models.ForeignKey(to='karaage.Machine')),
            ],
            options={
                'db_table': 'cache_machinecache',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MachineCategoryCache',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date', models.DateField(auto_now_add=True)),
                ('start', models.DateField()),
                ('end', models.DateField()),
                ('cpu_time', models.DecimalField(max_digits=30, decimal_places=2)),
                ('no_jobs', models.IntegerField()),
                ('available_time', models.DecimalField(max_digits=30, decimal_places=2)),
                ('machine_category', models.ForeignKey(to='karaage.MachineCategory')),
            ],
            options={
                'db_table': 'cache_machinecategorycache',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='PersonCache',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date', models.DateField(auto_now_add=True)),
                ('start', models.DateField()),
                ('end', models.DateField()),
                ('cpu_time', models.DecimalField(max_digits=30, decimal_places=2)),
                ('no_jobs', models.IntegerField()),
                ('machine_category', models.ForeignKey(to='karaage.MachineCategory')),
                ('person', models.ForeignKey(to='karaage.Person')),
                ('project', models.ForeignKey(to='karaage.Project')),
            ],
            options={
                'db_table': 'cache_personcache',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='ProjectCache',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('date', models.DateField(auto_now_add=True)),
                ('start', models.DateField()),
                ('end', models.DateField()),
                ('cpu_time', models.DecimalField(max_digits=30, decimal_places=2)),
                ('no_jobs', models.IntegerField()),
                ('machine_category', models.ForeignKey(to='karaage.MachineCategory')),
                ('project', models.ForeignKey(to='karaage.Project')),
            ],
            options={
                'db_table': 'cache_projectcache',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Queue',
            fields=[
                ('name', models.CharField(max_length=50, serialize=False, primary_key=True)),
                ('description', models.CharField(max_length=200, null=True, blank=True)),
            ],
            options={
                'db_table': 'queue',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='UsedModules',
            fields=[
                ('jobid', models.CharField(max_length=100, serialize=False, primary_key=True)),
                ('date_added', models.DateField(auto_now_add=True)),
                ('modules', models.TextField()),
            ],
            options={
                'db_table': 'usage_usedmodules',
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='projectcache',
            unique_together=set([('date', 'start', 'end', 'project', 'machine_category')]),
        ),
        migrations.AlterUniqueTogether(
            name='personcache',
            unique_together=set([('date', 'start', 'end', 'person', 'project', 'machine_category')]),
        ),
        migrations.AlterUniqueTogether(
            name='machinecategorycache',
            unique_together=set([('date', 'start', 'end', 'machine_category')]),
        ),
        migrations.AlterUniqueTogether(
            name='machinecache',
            unique_together=set([('date', 'start', 'end', 'machine')]),
        ),
        migrations.AlterUniqueTogether(
            name='institutecache',
            unique_together=set([('date', 'start', 'end', 'institute', 'machine_category')]),
        ),
        migrations.AddField(
            model_name='cpujob',
            name='queue',
            field=models.ForeignKey(blank=True, to='kgusage.Queue', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='cpujob',
            name='software',
            field=models.ManyToManyField(to='kgsoftware.SoftwareVersion', null=True, blank=True),
            preserve_default=True,
        ),
    ]
